// ALL INFORMATION FOR PARSING HATP DOMAIN AND SOCIAL RULES
//------------------------------------------ HEADER ------------------------------------------
header {
#include<iostream>
#include<fstream>
#include<string>
#include<vector>
#include<set>
#include<map>
#include<boost/algorithm/string/replace.hpp>
#include<boost/algorithm/string/regex.hpp>
#include<boost/regex.hpp>

#include <HATPParserException.hh>
#include <Entity.hh>
#include <WorldBaseReport.hh>

//#define GEN_DIRECTORY "generated/"
#define WORLDBASE_EXIT_FILE "initWorldBase.cc"
#define HTN_EXIT_FILE "initHTNBase.cc"
#define SOCIALRULEBASE_EXIT_FILE "initSocialRuleBase.cc"
#define SOCIALRULES_EXIT_FILE "socialRules.cc"
#define USR_FUNC_EXIT_FILE "userFunc.cc"
#define SUFFIX_TASK_FILE "Functions.cc"
#define BAD_DECOMPO_RULE "bad_decompo"
#define TIME_PART_TAG "TIME_PART"
#define EFFORT_BALANCING_TAG "EFFORT_BALANCING_PART"
#define INTRICACY_CONTROL_TAG "INTRICACY_CONTROL_PART"
#define WASTED_TIME_TAG "WASTED_TIME_PART"

#define STATIC_ETYPE "static"
#define DYNAMIC_ETYPE "dynamic"
#define ATOM_ETYPE "atom"
#define SET_ETYPE "set"
#define NUMBER_ETYPE "number"
#define STRING_ETYPE "string"
#define BOOL_ETYPE "bool"
#define NULL_ETYPE "NULL"
#define KNOWN_ETYPE "known"
#define UNKNOWN_ETYPE "unknown"
#define ENTITY_ETYPE "Entity"
#define ATTR_ETYPE "Attribute"

#define INFORMATION_TAG "information"
#define CONTRADICTION_TAG "contradiction"
#define QUESTION_TAG "question"

}

//------------------------------------------ Options ------------------------------------------
options {
	language="Cpp";
	genHashLines = true; // to get line numbers
}

//------------------------------------------ Parser ------------------------------------------
{
	#include<boost/lexical_cast.hpp> // for string->number conversion
	#include "tools.hh" // for useful operators (>>, ...)
}

class HATPParser extends Parser;

//******** Parser options ********
options {
	k = 15;							// lookahead
	codeGenMakeSwitchThreshold = 2;	// use of switch command instead of "if...else"
	codeGenBitsetTestThreshold = 5;	// optimization of code by exploring 5 bitsets ahead
	defaultErrorHandler = false;		// exceptions will be generated by HATP progammer
	buildAST = false;					// no needs of an Abstract Syntax Tree

}

//******** Parser fields ********
{
	std::string outputDirectory;

	// FOR FACT DATABASE C++ CODE
	std::map<std::string, bool> definedEntityTypes;
	std::map<std::string, std::map<std::string, std::vector<std::string> > > entityTypesAttributes;
	std::map<std::string, std::string> entities;
	// FOR HTN C++ CODE
	std::string current_task_name;
	std::map<std::string, std::string> current_task_parameters;
	std::vector<std::pair<std::string, std::string> > current_task_select_terms;
	std::vector<std::string> current_task_select_terms_code;
	std::vector<bool> current_task_select_once;
	std::set<unsigned int> current_decomposition_nodes;
	std::multimap<int, std::pair<std::string, std::vector<std::string> > > tasks_in_decompositions;
	std::map<std::string, std::vector<std::string> > htnTasks;
	std::map<std::string, bool> htnTasksIsAction;
	std::map<std::string, std::string> htnTasksCode;
	// FOR BELIEF MANAGEMENT
	bool myselfAlreadyDefined, beliefMethodDefined;
	std::string myselfEntity;
	std::map<std::string, std::string> commActionNameList;
	std::map<std::string, std::string> commActionDefaultList;
	std::map<std::string, std::string> commActionRedefinedList;
	// FOR FORALL MANAGEMENT IN PRECONDS
	bool useFORALLinPreconds;
	std::map<std::string, std::pair<std::string, std::string> >contentFORALLinPreconds;
	// FOR SOCIAL RULE BASE C++ CODE
	std::multimap<unsigned int,unsigned int> badDecompositionNodes;
	std::map<std::string, int> priorities;
	// TOOLS
	void completeMapWithEntities(std::map<std::string, std::pair<std::string, std::string> >& inputMap) {
		for(std::map<std::string, std::string>::const_iterator it=entities.begin(); it != entities.end() ; ++it){
			inputMap[it->first] = std::make_pair(it->second, "wb.find(\""+ it->first +"\")");
		}
	}
	void completeMapWithTaskParameters(std::map<std::string, std::pair<std::string, std::string> >& inputMap) {
		for(std::map<std::string, std::string>::const_iterator it=current_task_parameters.begin(); it != current_task_parameters.end() ; ++it){
			inputMap[it->first] = std::make_pair(it->second, it->first);
		}
	}
	void completeMapWithTaskSelectTerms(std::map<std::string, std::pair<std::string, std::string> >& inputMap) {
		for(unsigned int i=0 ; i<current_task_select_terms.size() ; ++i){
			inputMap[current_task_select_terms[i].first] = std::make_pair(current_task_select_terms[i].second, current_task_select_terms_code[i]);
		}
	}
	public: void setOutput(const std::string& outputDirectory){
		this->outputDirectory=outputDirectory;
	}
}

//******** start hatpFile rule ********
hatpFile throws HATPParserException : {
	std::string wbCode, srCode;
	std::pair<std::string, std::string> initAndMainSRBCode;
	std::map<std::string, std::vector<std::string> >::const_iterator itTasks;
	std::map<std::string, std::string>::const_iterator bmTasks;
}
wbCode=worldbaseInit htnDef initAndMainSRBCode=socialRulesDef EOF {
	std::string exitFile;
	std::string errorMsg;
//	exitFile = GEN_DIRECTORY;
	exitFile = outputDirectory;
	exitFile += WORLDBASE_EXIT_FILE;
	std::ofstream worldbaseFileToWrite(exitFile.c_str(), std::ios::out | std::ios::trunc);
	if(worldbaseFileToWrite) {
		worldbaseFileToWrite << wbCode << std::endl;
	} else { // we have failed when opening the file, we throw an exception
		errorMsg = "when trying to write C++ code of fact database initialization, failure when opening file " + exitFile + ".";
		throw HATPParserException(ERROR, 0, errorMsg);
	}
	worldbaseFileToWrite.close();
	std::cout << "File for fact database initialization written." << std::endl;

	// we write C++ code for all tasks
	for(itTasks = htnTasks.begin() ; itTasks!=htnTasks.end() ; ++itTasks) {
//		exitFile = GEN_DIRECTORY;
		exitFile = outputDirectory;
		exitFile += itTasks->first + SUFFIX_TASK_FILE;
		std::ofstream taskFile(exitFile.c_str(), std::ios::out | std::ios::trunc);
		if(taskFile) {
			taskFile << htnTasksCode[itTasks->first] << std::endl;
		} else { // we have failed when opening the file, we throw an exception
			errorMsg = "when trying to write C++ code of a task file, failure when opening file " + exitFile + ".";
			throw HATPParserException(ERROR, 0, errorMsg);
		}
		taskFile.close();
	}
	std::cout << "Files for HTN tasks written." << std::endl;

	// code to write Commtask files
	if(myselfAlreadyDefined && !beliefMethodDefined) {
		errorMsg = "No belief Management main Method defined, whereas Myself is defined!";
		throw HATPParserException(ERROR, 0, errorMsg);

	}
	std::string commActContent = "";
	for(bmTasks = commActionNameList.begin() ; bmTasks != commActionNameList.end() ; ++bmTasks) {
		if(commActionRedefinedList.find(bmTasks->second) != commActionRedefinedList.end()){
			commActContent = commActionRedefinedList.find(bmTasks->second)->second;
		} else {
			commActContent = commActionDefaultList.find(bmTasks->second)->second;
		}
//		exitFile = GEN_DIRECTORY;
		exitFile = outputDirectory;
		exitFile += bmTasks->second + SUFFIX_TASK_FILE;
		std::ofstream bmTaskFile(exitFile.c_str(), std::ios::out | std::ios::trunc);
		if(bmTaskFile){
			bmTaskFile << commActContent << std::endl;
		} else {
			errorMsg = "when trying to write C++ code of a communication task file, failure when opening file " + exitFile + ".";
			throw HATPParserException(ERROR, 0, errorMsg);
		}
		bmTaskFile.close();
	}
	std::cout << "Files for Belief Management written." << std::endl;

	// we make C++ code for initHTNBase function, then we write in outputDirectory/HTN_EXIT_FILE
	std::string initHTNBaseCode = "#include<";
	initHTNBaseCode += USR_FUNC_EXIT_FILE;
	initHTNBaseCode += ">\n";
	// we explore htnTasks and commTasks to include headers
	for(itTasks = htnTasks.begin() ; itTasks != htnTasks.end() ; ++itTasks){
		initHTNBaseCode += "#include<" + itTasks->first + SUFFIX_TASK_FILE + ">\n";
	}
	for(bmTasks = commActionNameList.begin() ; bmTasks != commActionNameList.end() ; ++bmTasks){
		initHTNBaseCode += "#include<" + bmTasks->second + SUFFIX_TASK_FILE + ">\n";
	}
	initHTNBaseCode += "\nusing namespace std;\n\nvoid initHTNBase(HTNBase& taskBase)\n{\nvector<string> taskTypes;\n\n";
	std::vector<std::string> argTypes;
	unsigned int i;
	for(itTasks = htnTasks.begin() ; itTasks != htnTasks.end() ; ++itTasks) { // we explore htnTasks to include headers
		initHTNBaseCode += "// task " + itTasks->first + "\ntaskTypes.clear();\n";
		argTypes = itTasks->second;
		for(i=0 ; i<argTypes.size() ; ++i){
			initHTNBaseCode += "taskTypes.push_back(\"" + argTypes[i] + "\");\n";
		}
		initHTNBaseCode += "Task* " + itTasks->first + " = new Task(\""+ itTasks->first +"\", taskTypes, ";
		if(htnTasksIsAction[itTasks->first] == true) {
			initHTNBaseCode += "ACTION, "+ itTasks->first +"Precond, "+ itTasks->first +"PrecondWithReports, NULL, NULL, "+ itTasks->first +"Effects, " + itTasks->first +"EffectsWithReports, "+ itTasks->first +"Cost, "+ itTasks->first+"Time);\n";
		} else {
			initHTNBaseCode += "METHOD, NULL, NULL, " + itTasks->first + "Empty, " + itTasks->first + "Decompo, NULL, NULL, NULL, NULL);\n";
		}
		initHTNBaseCode += "taskBase.add(" + itTasks->first + ");\n\n";
	}

	// Code to write belief management in HTN file
	initHTNBaseCode += "// Belief Management\n";
	initHTNBaseCode += "BeliefMethod* bm = new BeliefMethod();\n\n";

	for(bmTasks = commActionNameList.begin() ; bmTasks != commActionNameList.end() ; ++bmTasks) {
		initHTNBaseCode += "// task " + bmTasks->second + "\ntaskTypes.clear();\n";
		initHTNBaseCode += "taskTypes.push_back(\"Agent\");\n";
		initHTNBaseCode += "taskTypes.push_back(\"Agent\");\n";
		initHTNBaseCode += "taskTypes.push_back(\"Entity\");\n";
		initHTNBaseCode += "taskTypes.push_back(\"Attribute\");\n";
		initHTNBaseCode += "Task* " + bmTasks->second + " = new Task(" + "\"" + bmTasks->second + "\", taskTypes, ";
		initHTNBaseCode += "ACTION, " + bmTasks->second +"Precond, NULL, " + "NULL, NULL, ";
		initHTNBaseCode += bmTasks->second +"Effects, NULL, " + bmTasks->second +"Cost, " + bmTasks->second +"Time);\n";
		initHTNBaseCode += "bm->addCommTask(\"" + bmTasks->first + "\", " + bmTasks->second + ");\n";
		initHTNBaseCode += "taskBase.add(" + bmTasks->second + ");\n\n";
	}
	initHTNBaseCode += "taskBase.setBeliefMethod(bm);\n\n";

	// end HTNBase code
	initHTNBaseCode += "}\n";
//	exitFile = GEN_DIRECTORY;
	exitFile = outputDirectory;
	exitFile += HTN_EXIT_FILE;
	std::ofstream htnFileToWrite(exitFile.c_str(), std::ios::out | std::ios::trunc);
	if(htnFileToWrite){
		htnFileToWrite << initHTNBaseCode << std::endl;
	} else { // we have failed when opening the file, we throw an exception
		errorMsg = "when trying to write C++ code of HTNbase initialization, failure when opening file " + exitFile + ".";
		throw HATPParserException(ERROR, 0, errorMsg);
	}
	htnFileToWrite.close();
	std::cout << "File for HTN initialization written." << std::endl;
	// we write social_rules code in outputDirectory/SOCIALRULES_EXIT_FILE (we overwrite the file if it exists)
//	exitFile = GEN_DIRECTORY;
	exitFile = outputDirectory;
	exitFile += SOCIALRULES_EXIT_FILE;
	std::ofstream socialrulesFileToWrite(exitFile.c_str(), std::ios::out | std::ios::trunc);
	if(socialrulesFileToWrite){
		socialrulesFileToWrite << initAndMainSRBCode.second << std::endl;
	} else { // we have failed when opening the file, we throw an exception
		errorMsg = "when trying to write C++ code of fact database initialization, failure when opening file " + exitFile + ".";
		throw HATPParserException(ERROR, 0, errorMsg);
	}
	socialrulesFileToWrite.close();
	std::cout << "File for social rules written." << std::endl;
	// we write initSocialRuleBaseCode in outputDirectory/SOCIALRULEBASE_EXIT_FILE (we overwrite the file if it exists)
//	exitFile = GEN_DIRECTORY;
	exitFile = outputDirectory;
	exitFile += SOCIALRULEBASE_EXIT_FILE;
	std::ofstream socialrulebaseFileToWrite(exitFile.c_str(), std::ios::out | std::ios::trunc);
	if(socialrulebaseFileToWrite){
		socialrulebaseFileToWrite << initAndMainSRBCode.first << std::endl;
	} else { // we have failed when opening the file, we throw an exception
		errorMsg = "when trying to write C++ code of fact database initialization, failure when opening file " + exitFile + ".";
		throw HATPParserException(ERROR, 0, errorMsg);
	}
	socialrulebaseFileToWrite.close();
	std::cout << "File for social rule base initialization written." << std::endl;
} ;
//******** end hatpFile rule ********

//******* start worldbaseInit rule *******
//syntax: factdatabase {....} return CPPcode for worldbase initialization
worldbaseInit returns[std::string CPPcode] throws HATPParserException : {
	definedEntityTypes[AGENT] = false;
	entityTypesAttributes[AGENT] = std::map<std::string, std::vector<std::string> >();
	myselfAlreadyDefined = false;
	beliefMethodDefined = false;
}
FACTDATABASE LEFT_SCOPE
(definitionEntityType)* (definitionEntityAttributes)* (newEntity)* {
	std::map<std::string, std::pair<std::string, std::string> > tmpMap;
	this->completeMapWithEntities(tmpMap);
	std::pair<std::string, std::string> effectsCodeWithAndWithoutReports;
}
effectsCodeWithAndWithoutReports=effects[tmpMap,true,true]
	end:RIGHT_SCOPE {
	CPPcode = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n";
	CPPcode += "void initWorldBase(WorldBase& wb)\n{\n";
	CPPcode += "// worldbase structure\n\n";
	CPPcode += "vector<string> sNumbers, dNumbers, ssNumbers, dsNumbers,\n";
	CPPcode += "\tsStrings, dStrings, ssStrings, dsStrings,\n";
	CPPcode += "\tsBools, dBools, ssBools, dsBools,\n";
	CPPcode += "\tsEntities, dEntities, ssEntities, dsEntities;\n\n";
	// we generate code by entity types, so we get all needed entity types
	std::set<std::string> setEntityTypes;
	std::map<std::string, std::string>::iterator itDE;
	for(itDE = entities.begin() ; itDE != entities.end() ; ++itDE){
		setEntityTypes.insert(itDE->second);
	}
	std::set<std::string>::iterator itET;
	for(itET = setEntityTypes.begin() ; itET != setEntityTypes.end() ; ++itET) {
		CPPcode += "// " + *itET + "(s)\n";
		std::map<std::string, bool>::iterator it_def = definedEntityTypes.find(*itET);
		if(it_def == definedEntityTypes.end()) { // for security, normally useless
			throw HATPParserException(ERROR, end->getLine(), "when creating C++ code for initWorldBase, entity type " + *itET + " unknown");
		}
		else if (it_def->second==true) {// we add code only if some attributes have been defined
			std::map<std::string, std::vector<std::string> > attrs = entityTypesAttributes[*itET];
			for(std::map<std::string, std::vector<std::string> >::const_iterator itAttr = attrs.begin(); itAttr != attrs.end() ; ++itAttr) {
				if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == NUMBER_ETYPE) {
					CPPcode+="sNumbers.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == NUMBER_ETYPE) {
					CPPcode+="dNumbers.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == NUMBER_ETYPE) {
					CPPcode+="ssNumbers.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == NUMBER_ETYPE) {
					CPPcode+="dsNumbers.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == STRING_ETYPE) {
					CPPcode+="sStrings.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == STRING_ETYPE) {
					CPPcode+="dStrings.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == STRING_ETYPE) {
					CPPcode+="ssStrings.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == STRING_ETYPE) {
					CPPcode+="dsStrings.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == BOOL_ETYPE) {
					CPPcode+="sBools.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == ATOM_ETYPE && itAttr->second[2] == BOOL_ETYPE) {
					CPPcode+="dBools.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == BOOL_ETYPE) {
					CPPcode+="ssBools.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == SET_ETYPE && itAttr->second[2] == BOOL_ETYPE) {
					CPPcode+="dsBools.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == ATOM_ETYPE) {
					CPPcode+="sEntities.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == ATOM_ETYPE) {
					CPPcode+="dEntities.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == STATIC_ETYPE && itAttr->second[1] == SET_ETYPE) {
					CPPcode+="ssEntities.push_back(\""+ itAttr->first +"\");\n";
				} else if(itAttr->second[0] == DYNAMIC_ETYPE && itAttr->second[1] == SET_ETYPE) {
					CPPcode+="dsEntities.push_back(\""+ itAttr->first +"\");\n";
				} else {
					std::cerr << "ERROR: unrecognized type when generating code for factdatabase initialization" << std::endl;
				}
			}
			CPPcode += "\n";
		}
		// code for entities creation
		for(itDE = entities.begin() ; itDE != entities.end() ; ++itDE) { // we explore worldbase entities
			if(itDE->second == *itET) { // if the entity is of current type
				CPPcode += "wb.add(Entity(\"" + itDE->first + "\", \"";
				CPPcode += itDE->second + "\", sNumbers, dNumbers, ssNumbers, dsNumbers, sStrings, dStrings,";
				CPPcode += " ssStrings, dsStrings, sBools, dBools, ssBools, dsBools, sEntities, dEntities,";
				CPPcode += "ssEntities, dsEntities));\n";
			}
		}
		CPPcode += "\n";
		// code to free memory
		CPPcode += "sNumbers.clear(); dNumbers.clear(); ssNumbers.clear(); dsNumbers.clear();\n";
		CPPcode += "sStrings.clear(); dStrings.clear(); ssStrings.clear(); dsStrings.clear();\n";
		CPPcode += "sBools.clear(); dBools.clear(); ssBools.clear(); dsBools.clear();\n";
		CPPcode += "sEntities.clear(); dEntities.clear(); ssEntities.clear(); dsEntities.clear();\n\n";
	} // end for all types
	// code for belief management initialization
	if(myselfAlreadyDefined) {
		CPPcode += "// Belief Management init\n";
		CPPcode += "wb.setMyselfName(\"" + myselfEntity + "\");\n\n";
	}
	// end of factdatabase init
	CPPcode += "// worldbase init\n\n"+ effectsCodeWithAndWithoutReports.second + "}\n";
	std::cout << "Fact database initialization done." << std::endl;
} ;
//******** end worldbaseInit rule ********

//******* start definitionEntityType rule *******
//syntax: define entity_type <name>;
definitionEntityType throws HATPParserException :
DEFINE ENTITY_TYPE name:ENTITY_ID {
	// if there is already an entity_type <name> throw an exception
	if(definedEntityTypes.find(name->getText()) != definedEntityTypes.end()) {
		throw HATPParserException(ERROR, name->getLine(), "entity type " + name->getText() + " previously declared");
	} else {
		definedEntityTypes[name->getText()] = false; // declaration of entity type <name>
		entityTypesAttributes[name->getText()] = std::map<std::string, std::vector<std::string> >();
	}
}
SEMICOLON ;
//******** end definitionEntityType rule ********

//******* start definitionEntityAttributes rule *******
//syntax: define entity_attributes <name> {...}
definitionEntityAttributes throws HATPParserException :
DEFINE ENTITY_ATTRIBUTES name:ENTITY_ID {
	std::map<std::string, bool>::iterator it = definedEntityTypes.find(name->getText());
	if(it == definedEntityTypes.end()) {// no associated entity type defined
		throw HATPParserException(ERROR, name->getLine(), "trying to define attributes of unknown entity type " + name->getText());
	} else if(it->second == true) { // attributes has already been defined
	throw HATPParserException(ERROR, name->getLine(), "attributes of entity type " + it->first + " has been previously declared");
	} else {
		it->second = true; // everything OK
	}
}
LEFT_SCOPE (defSingleAttribute[name->getText()])* RIGHT_SCOPE ;
//******** end definitionEntityAttributes rule ********

//******* start defSingleAttribute rule *******
//syntax: static/dynamic atom/set number/string/bool/entity <name>;
defSingleAttribute[std::string entityType] throws HATPParserException : {
	std::vector<std::string> vec(3);
}
( STATIC {vec[0] = STATIC_ETYPE;} | DYNAMIC {vec[0] = DYNAMIC_ETYPE;} )
( ATOM {vec[1] = ATOM_ETYPE;} | SET {vec[1] = SET_ETYPE;} )
( NUMBER {vec[2] = NUMBER_ETYPE;} | STRING {vec[2] = STRING_ETYPE;} | BOOL {vec[2] = BOOL_ETYPE;}
| typeName:ENTITY_ID {
	// typeName must be a declared entity type
	if(definedEntityTypes.find(typeName->getText()) == definedEntityTypes.end()) {
		throw HATPParserException(ERROR, typeName->getLine(),"entity type \"" + typeName->getText()+ "\" undeclared");
	} else {
		vec[2] = typeName->getText();
	}
		}
)
	name:OTHER_ID {
	if(entityTypesAttributes[entityType].find(name->getText()) != entityTypesAttributes[entityType].end()) {
		throw HATPParserException(ERROR, name->getLine(),"entity type " + entityType + " has already an attribute of name " + name->getText());
	} else {
		entityTypesAttributes[entityType][name->getText()] = vec;
	}
}
SEMICOLON ;
//******* start defSingleAttribute rule *******

//******* start newEntity rule *******
//syntax: <name> = new <entity type>; or <name> = myself; (belief management)
newEntity throws HATPParserException :
	eName:ENTITY_ID EQUAL {
		std::map<std::string, std::string>::iterator it = entities.find(eName->getText());
	}
	(NEW eType:ENTITY_ID {
		if(it != entities.end()) {// entity already defined
			throw HATPParserException(ERROR, eName->getLine(), "entity " + eName->getText() + " already exists in the fact database");
		} else if(definedEntityTypes.find(eType->getText()) == definedEntityTypes.end()) { // entity type is unknown
			throw HATPParserException(ERROR, eName->getLine(), "entity type " + eType->getText() + " is unknown");
		} else { // everything is OK
			entities[eName->getText()] = eType->getText();
		}
	}
	| MYSELF {
		if(myselfAlreadyDefined) {
			throw HATPParserException(ERROR, eName->getLine(), "Agent myself as already been defined");
		} else if(it != entities.end()) {
			throw HATPParserException(ERROR, eName->getLine(), "entity " + eName->getText() + " already exists in the fact database");
		} else {
			entities[eName->getText()] = AGENT;
			myselfEntity = eName->getText();
			myselfAlreadyDefined = true;
		}
	})
SEMICOLON ;
//******** end newEntity rule ********

//******** start singleTerm rule ********
// availableEntities contains names of available entities associated to their type and CPP code
// cppEntityName is the CPP code to access to the entity name if the term is an entity attribute
// attributeName is the name of the attribute if the term is an entity attribute
singleTerm[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase,
	std::string& cppEntityName, std::string& attributeName]
	returns[std::pair<std::vector<std::string>, std::string> typeAndCppCode] throws HATPParserException {
	std::vector<std::string> vec(4);
	std::string tmpCode;
} : {
	std::map<std::string, std::vector<std::string> > attrs;
	std::map<std::string, std::vector<std::string> >::const_iterator itAttrs;
}
//the singleTerm is an entity or an attribute
	name:ENTITY_ID {
	cppEntityName = "";
	//name must be in entities
	std::map<std::string, std::pair<std::string, std::string> >::const_iterator it;
	it = availableEntities.find(name->getText());
	if(it == availableEntities.end()) {
		throw HATPParserException(ERROR, name->getLine(), "entity \"" + name->getText() + "\" unknown");
	} else {
		vec[0] = DYNAMIC_ETYPE;
		vec[1] = ATOM_ETYPE;
		vec[2] = (it->second).first;
		vec[3] = ""; // init agent name for belief management
		typeAndCppCode = std::make_pair(vec, (it->second).second);
	}
}
// concerned agent (belief management)
(LEFT_PAREN bName:ENTITY_ID RIGHT_PAREN {
	if(initPhase == false) {
		//throw HATPParserException(ERROR, bName->getLine(), "Effects for other agent than myself not authorized");
		// a faire test si commAction ... ??
		vec[3] = bName->getText();
		cppEntityName = name->getText();
		if(vec[3] != "Myself") {
			typeAndCppCode = std::make_pair(vec, "wb.findForAgent("+vec[3]+"->getName(), "+cppEntityName+"->getName())");
		} else {
			typeAndCppCode = std::make_pair(vec, "wb.findForAgent(\"Myself\", "+cppEntityName+"->getName())");
		}
	} else {
		vec[3] = bName->getText();
		cppEntityName = name->getText();
		typeAndCppCode = std::make_pair(vec, "wb.initForAgent(\""+vec[3]+"\", \""+cppEntityName+"\")");
	}
})*

// dot attribut
( DOT
| (attbm:ENTITY_ID {
	// case for bm
	attributeName = attbm->getText();
} )
| ( att:OTHER_ID {
	if(cppEntityName == "") {
		cppEntityName = typeAndCppCode.second;
	}
	attributeName = att->getText();
	// if vec[2] is not an atom Entity
	if( vec[1] != ATOM_ETYPE || vec[2] == NUMBER_ETYPE || vec[2] == STRING_ETYPE || vec[2] == BOOL_ETYPE ) {
		throw HATPParserException(ERROR, att->getLine(), "when trying to access to an attribute with \""
		+ att->getText() + "\", previous part of the term is not an atom Entity");
	} else { // attr must be in attributes of "name" type
		attrs = entityTypesAttributes[vec[2]];
		itAttrs = attrs.find(att->getText());
		if(itAttrs == attrs.end()) {
			throw HATPParserException(ERROR, att->getLine(), "entity type \"" + vec[2] + "\" has no attribute \"" + att->getText() + "\"");
		} else {
			vec[0] = (itAttrs->second)[0];
			vec[1] = (itAttrs->second)[1];
			vec[2] = (itAttrs->second)[2];
			tmpCode = typeAndCppCode.second+"->get";
			if(vec[1] == ATOM_ETYPE && vec[2] == NUMBER_ETYPE) {
				tmpCode += "<number>(\""+ att->getText() +"\")";
			} else if(vec[1] == ATOM_ETYPE && vec[2] == STRING_ETYPE) {
				tmpCode += "<string>(\""+ att->getText() +"\")";
			} else if(vec[1] == ATOM_ETYPE && vec[2] == BOOL_ETYPE) {
				tmpCode += "<bool>(\""+ att->getText() +"\")";
			} else if(vec[1] == ATOM_ETYPE) { // attribute is an Entity
				if(initPhase) {
					tmpCode += "<Entity*>(\""+ att->getText() +"\")";
				} else {
					tmpCode += "<LightEntity*>(\""+ att->getText() +"\")";
				}
			} else if(vec[1] == SET_ETYPE && vec[2] == NUMBER_ETYPE) {
				tmpCode += "< set<number> >(\""+ att->getText() +"\")";
			} else if(vec[1] == SET_ETYPE && vec[2] == STRING_ETYPE) {
				tmpCode += "< set<string> >(\""+ att->getText() +"\")";
			} else if(vec[1] == SET_ETYPE && vec[2] == BOOL_ETYPE) {
				tmpCode += "< set<bool> >(\""+ att->getText() +"\")";
			} else if(vec[1] == SET_ETYPE) { // attribute is an Entity
				if(initPhase) {
					tmpCode += "< set<Entity*> >(\""+ att->getText() +"\")";
				} else {
					tmpCode += "< set<LightEntity*> >(\""+ att->getText() +"\")";
				}
			}
			typeAndCppCode = std::make_pair(vec, tmpCode);
		}
	}
}
| sSize:SIZE LEFT_PAREN RIGHT_PAREN {
	if( vec[1] != SET_ETYPE ) {
		throw HATPParserException(ERROR, sSize->getLine(), " \".size()\" must be used on a set");
	} else {
		vec[0] = STATIC_ETYPE;
		vec[1] = ATOM_ETYPE;
		vec[2] = NUMBER_ETYPE;
		cppEntityName = "";
		attributeName = "";
		typeAndCppCode = std::make_pair(vec, tmpCode+".size()");
	}
}
) )*
| nullp:NULL_POINTER { //the singleTerm is a NULL Pointer
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = NULL_ETYPE;
	typeAndCppCode = std::make_pair(vec, nullp->getText());
}
| val:NUMBER_ID { //the singleTerm is a number
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = NUMBER_ETYPE;
	typeAndCppCode = std::make_pair(vec, val->getText());
}
| str:STRING_ID { //the singleTerm is a string
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = STRING_ETYPE;
	typeAndCppCode = std::make_pair(vec, "string("+str->getText()+")");
}
| UNKNOWN { // the singleTerm is unknown
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = UNKNOWN_ETYPE;
	typeAndCppCode = std::make_pair(vec, "");
}
| KNOWN { // the singleTerm is known
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = KNOWN_ETYPE;
	typeAndCppCode = std::make_pair(vec, "");
}
| bStr:OTHER_ID { //the singleTerm is a bool
	if( !(bStr->getText() == "true" || bStr->getText() == "false") ) {
		throw HATPParserException(ERROR, bStr->getLine(), "unrecognized text \""+ bStr->getText() +"\"");
	} else {
		vec[0] = STATIC_ETYPE;
		vec[1] = ATOM_ETYPE;
		vec[2] = BOOL_ETYPE;
		typeAndCppCode = std::make_pair(vec, bStr->getText());
	}
}
| (fStr:OTHER_ID {} leftp:LEFT_PAREN typeAndCppCode=evalPredicateArguments[availableEntities,leftp->getLine()] {} RIGHT_PAREN {
	//code += typeAndCppCode.second;
	// we store taskName and its arguments types (if they have been identified) to ensure HTN consistency after all tasks have been defined
	//std::pair<std::string, std::vector<std::string> > args = std::make_pair(taskName->getText(), argsTypesAndCPPcode.first);
	//tasks_in_decompositions.insert( std::make_pair(taskName->getLine(), args) );
	//}
	//RIGHT_PAREN

	//else {
	vec[0] = STATIC_ETYPE;
	vec[1] = ATOM_ETYPE;
	vec[2] = BOOL_ETYPE;
	//typeAndCppCode = std::make_pair(vec, typeAndCppCode.second);
	typeAndCppCode = std::make_pair(vec, fStr->getText()+"("+typeAndCppCode.second+")");

	//std::pair<std::string, std::vector<std::string> > args = std::make_pair(fStr->getText(), argsTypesAndCPPcode.first);

	//}
 }) ;
//******** end singleTerm rule ********

//******* start effects rule *******
effects[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase, bool canBeStatic]
returns [std::pair<std::string, std::string> codeWithAndWithoutReports] throws HATPParserException : {
	std::string code1, code2;
	std::pair<std::string, std::string> tmpPair;
}
(tmpPair=effect[availableEntities, initPhase, canBeStatic] {
	code1 += tmpPair.first;
	code2 += tmpPair.second;
}
)* { codeWithAndWithoutReports = std::make_pair(code1, code2); } ;
//******** end effects rule ********

//******* start effect rule *******
effect[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase, bool canBeStatic]
returns [std::pair<std::string, std::string> codeWithAndWithoutReports] throws HATPParserException {
	std::pair<std::string, std::string> tmpPair;
	codeWithAndWithoutReports = std::make_pair("","");
} :
//alternative 1
tmpPair=effectTerm[availableEntities, initPhase, canBeStatic] {
	codeWithAndWithoutReports.first += tmpPair.first+"\n";
	codeWithAndWithoutReports.second += tmpPair.second+"\n";
}
//alternative 2 (if)
| { std::string tmp; contentFORALLinPreconds.clear(); useFORALLinPreconds = false; }
IF LEFT_SCOPE tmpPair=preconds[availableEntities, initPhase] RIGHT_SCOPE {
	std::map<std::string, std::pair<std::string, std::string > >::iterator itFORALLinPreconds;
	codeWithAndWithoutReports.first = "";
	codeWithAndWithoutReports.second = "";
	for(itFORALLinPreconds = contentFORALLinPreconds.begin() ; itFORALLinPreconds != contentFORALLinPreconds.end() ; itFORALLinPreconds++) {
		codeWithAndWithoutReports.first += itFORALLinPreconds->second.first+"\n";
		codeWithAndWithoutReports.second += itFORALLinPreconds->second.second+"\n";
	}
	codeWithAndWithoutReports.first += "if(" + tmpPair.first + ")\n{\n";
	codeWithAndWithoutReports.second += "if(" + tmpPair.second + ")\n{\n";
}
LEFT_SCOPE (tmpPair=effect[availableEntities, initPhase, canBeStatic] {

	codeWithAndWithoutReports.first += tmpPair.first + "\n";
	codeWithAndWithoutReports.second += tmpPair.second +"\n";
}
)* {
	codeWithAndWithoutReports.first += "}";
	codeWithAndWithoutReports.second += "}";
}
RIGHT_SCOPE
//alternative 3 (call)
| CALL LEFT_PAREN tmpPair=effectcallTerm[availableEntities, initPhase, canBeStatic] RIGHT_PAREN SEMICOLON {
	codeWithAndWithoutReports.first += tmpPair.first+"\n";
	codeWithAndWithoutReports.second += tmpPair.second+"\n";
}
// alternative 4 (forall)
| FORALL LEFT_PAREN type:ENTITY_ID name: ENTITY_ID COMMA {
	std::ostringstream st;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities2;
	st << "vector<LightEntity*> " << name->getText() << " = wb.findByType(\"" << type->getText() << "\");\n";
	st << "for(unsigned int " << name->getText() << "_iterator=0 ; " << name->getText() << "_iterator < " << name->getText() <<".size() ;";
	st << "++" << name->getText() << "_iterator) {\n";
	codeWithAndWithoutReports.first += st.str();
	codeWithAndWithoutReports.second += st.str();
	std::map<std::string, std::pair<std::string, std::string> >::const_iterator it;
	for(it = availableEntities.begin() ; it != availableEntities.end() ; it++) {
		availableEntities2[it->first] = it->second;
	}
	availableEntities2[name->getText()]=std::make_pair(type->getText(), name->getText());
	std::string newEnt;
	newEnt = name->getText()+"["+name->getText()+"_iterator]";
}
LEFT_SCOPE tmpPair=preconds[availableEntities2, initPhase] RIGHT_SCOPE {
	boost::replace_all_regex(tmpPair.first, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	codeWithAndWithoutReports.first += "if("+ tmpPair.first + "){\n";
	codeWithAndWithoutReports.second += "if("+ tmpPair.second +"){\n";
}
COMMA LEFT_SCOPE (tmpPair=effect[availableEntities2, initPhase, canBeStatic] { // was effectTerm
	boost::replace_all_regex(tmpPair.first, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	codeWithAndWithoutReports.first += tmpPair.first + "\n";
	codeWithAndWithoutReports.second += tmpPair.second +"\n";
}
)*
RIGHT_SCOPE RIGHT_PAREN SEMICOLON {
	codeWithAndWithoutReports.first += "}\n}\n";
	codeWithAndWithoutReports.second += +"}\n}\n";
} ;
// end alternative
//******* end effect rule *******

//******** start effectTerm rule ********
// availableEntities contains names of available entities associated to their type and CPP code
// canBeStatic must be equal to true if static atoms can be modified
effectTerm[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase, bool canBeStatic]
returns [std::pair<std::string, std::string> codeWithAndWithoutReport] throws HATPParserException : {
	std::pair<std::vector<std::string>, std::string> termTypeAndCppCode1, termTypeAndCppCode2;
	std::string opCPPcode;
	unsigned int nbLine;
	std::string entityName1, attributeName1, entityName2, attributeName2;
}
termTypeAndCppCode1=singleTerm[availableEntities, initPhase, entityName1, attributeName1]
(op1:EQUAL { opCPPcode="="; nbLine=op1->getLine(); }
| op2:ADD_IN_SET { opCPPcode="<<="; nbLine=op2->getLine(); }
| op3:REMOVE_FROM_SET { opCPPcode=">>="; nbLine=op3->getLine(); }
)
termTypeAndCppCode2=singleTerm[availableEntities, initPhase, entityName2, attributeName2]
SEMICOLON {
	if( (opCPPcode == "=") // term1 and term2 must be atoms, term 1 must be a dynamic entity attribute they must be of same type
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE
	&& (canBeStatic || termTypeAndCppCode1.first[0] == DYNAMIC_ETYPE) &&
	(attributeName1 != "" || termTypeAndCppCode1.first[3] != "" ||
	termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE || termTypeAndCppCode2.first[2] == KNOWN_ETYPE)
	&& (termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2] ||
	( termTypeAndCppCode2.first[2] == NULL_ETYPE && (termTypeAndCppCode1.first[2] != NUMBER_ETYPE &&
	termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE) ) ||
	termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE || termTypeAndCppCode2.first[2] == KNOWN_ETYPE
	) ) ) {
		throw HATPParserException(ERROR, nbLine, "this effect must be done between a dynamic atom and a value of same kind");
	}
	if( (opCPPcode == "<<=" || opCPPcode == ">>=") // term1 must be a set and term2 must be an atom,
	// term 1 must be a dynamic entity attribute and they must be of the same type
	&& !( termTypeAndCppCode1.first[1] == SET_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE
	&& (canBeStatic || termTypeAndCppCode1.first[0] == DYNAMIC_ETYPE) && attributeName1 != ""
	&& termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2] ) ) {
		throw HATPParserException(ERROR, nbLine, "this effect must be done between a dynamic set and and a value of same kind");
	}
	// everything is OK
	std::string codeWithoutReport, codeWithReport;
	// for types number, string and bool
	if((termTypeAndCppCode1.first[2] == NUMBER_ETYPE || termTypeAndCppCode1.first[2] == STRING_ETYPE || termTypeAndCppCode1.first[2] == BOOL_ETYPE) &&
	(termTypeAndCppCode2.first[2] != KNOWN_ETYPE && termTypeAndCppCode2.first[2] != UNKNOWN_ETYPE)) {
		codeWithoutReport = termTypeAndCppCode1.second+" "+opCPPcode+" "+termTypeAndCppCode2.second+";";
		codeWithReport = "report.";
		if(opCPPcode == "=") {
			codeWithReport += "addAtomModifChange("+entityName1+"->getName(), \""+attributeName1+"\", "+termTypeAndCppCode1.second+", "+termTypeAndCppCode2.second+");\n";
		} else if( opCPPcode == "<<=" ) {
			codeWithReport += "addVectorModifAdd("+entityName1+"->getName(), \""+attributeName1+"\", "+termTypeAndCppCode2.second+");\n";
		} else { // opCPPcode is ">>="
			codeWithReport += "addVectorModifRemove("+entityName1+"->getName(), \""+attributeName1+"\", "+termTypeAndCppCode2.second+");\n";
		}
		codeWithReport += codeWithoutReport;
	} else { // for entities type
		// belief management ...
		if(termTypeAndCppCode1.first[3] != "") { // belief management, for an agent
			if(termTypeAndCppCode2.first[2] == KNOWN_ETYPE || termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE) {
				if(attributeName1 != "") {
					//entity(agent).attr = known / unknown
					if(termTypeAndCppCode2.first[2] == KNOWN_ETYPE) {
						codeWithoutReport = "wb.initForAgent(\""+termTypeAndCppCode1.first[3]+"\", \""+
						entityName1+"\")->setAttrKnown(\""+attributeName1+"\", true);";
					}
					if(termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE) {
						codeWithoutReport = "wb.initForAgent(\""+termTypeAndCppCode1.first[3]+"\", \""+
						entityName1+"\")->setAttrUnknown(\""+attributeName1+"\", true);";
					}
				} else {
					// entity(agent) = known / unknown
					if(termTypeAndCppCode2.first[2] == KNOWN_ETYPE) {
						codeWithoutReport = "wb.initForAgent(\""+termTypeAndCppCode1.first[3]+"\", \""+entityName1+"\")->setKnown(true);";
					}
					if(termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE) {
						codeWithoutReport = "wb.initForAgent(\""+termTypeAndCppCode1.first[3]+"\", \""+entityName1+"\")->setUnknown(true);";
					}
				}
			} else {
				if(initPhase) {
					// entity(agent).attr = v
					codeWithoutReport = termTypeAndCppCode1.second+" "+opCPPcode+" "+termTypeAndCppCode2.second+";";
				} else {
					// effects of commAction
					if(attributeName1 != "") {
						codeWithoutReport = "wb.updateForAgent("+termTypeAndCppCode1.first[3]+"->getName(), "+termTypeAndCppCode2.second+", "+termTypeAndCppCode1.second+", "+attributeName1+"->getName());";
					} else {
						codeWithoutReport = "wb.updateForAgent("+termTypeAndCppCode1.first[3]+"->getName(), "+termTypeAndCppCode2.second+", "+termTypeAndCppCode1.second+", \"NULL\");";
					}
				}
			}
			codeWithReport += codeWithoutReport;
		} else if(termTypeAndCppCode2.first[2] == UNKNOWN_ETYPE) { // for myself

			if(attributeName1 != "") {
				codeWithoutReport = entityName1+"->setAttrUnknown(\""+attributeName1+"\", true);";
			} else {
				codeWithoutReport = termTypeAndCppCode1.second+"->setUnknown(true);";
			}
		// end belief management
		} else {
			codeWithoutReport = termTypeAndCppCode1.second+" "+opCPPcode+" "+termTypeAndCppCode2.second+";";
			codeWithReport = "report.";
			if(opCPPcode == "=") {
				if(termTypeAndCppCode2.first[2] == NULL_ETYPE) {
					codeWithReport += "addAtomModifChange("+entityName1+"->getName(), \""+attributeName1+"\", ("+
					termTypeAndCppCode1.second+")?("+termTypeAndCppCode1.second+
					"->getName()):string(\"NULL\"), string(\"NULL\"));\n";
				} else {
					codeWithReport += "addAtomModifChange("+entityName1+"->getName(), \""+attributeName1+"\", ("+
					termTypeAndCppCode1.second+")?("+termTypeAndCppCode1.second+"->getName()):string(\"NULL\"), ("+
					termTypeAndCppCode2.second+")?("+termTypeAndCppCode2.second+"->getName()):string(\"NULL\"));\n";
				}
			} else if( opCPPcode == "<<=" ) {
				codeWithReport += "addVectorModifAdd("+entityName1+"->getName(), \""+attributeName1+"\", ("+
				termTypeAndCppCode2.second+")?("+termTypeAndCppCode2.second+"->getName()):string(\"NULL\"));\n";
			} else { // opCPPcode is ">>="
				codeWithReport += "addVectorModifRemove("+entityName1+"->getName(), \""+attributeName1+"\", ("+
				termTypeAndCppCode2.second+")?("+termTypeAndCppCode2.second+"->getName()):string(\"NULL\"));\n";
			}
			codeWithReport += codeWithoutReport;
		}
	}

	codeWithAndWithoutReport = std::make_pair(codeWithReport, codeWithoutReport);
} ;
//******** end effectTerm rule ********

//******** start effectCallTerm rule ********
// availableEntities contains names of available entities associated to their type and CPP code
// canBeStatic must be equal to true if static atoms can be modified
effectcallTerm[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase,
bool canBeStatic]
returns [std::pair<std::string, std::string> codeWithAndWithoutReport] throws HATPParserException : {
	std::pair<std::vector<std::string>, std::string> termTypeAndCppCode1, termTypeAndCppCode2;
	std::string opCPPcode;
	unsigned int nbLine;
	std::string entityName1, attributeName1, entityName2, attributeName2;
}
termTypeAndCppCode1=singleTerm[availableEntities, initPhase, entityName1, attributeName1]
(op1:PLUS { opCPPcode="+"; nbLine=op1->getLine(); }
| op2:MINUS { opCPPcode="-"; nbLine=op2->getLine(); }
| op3:TIMES { opCPPcode="*"; nbLine=op3->getLine(); }
| op4:SLASH { opCPPcode="/"; nbLine=op4->getLine(); }
| op5:REMOVE_FROM_SET { opCPPcode=">>="; nbLine=op5->getLine(); }
)
termTypeAndCppCode2=singleTerm[availableEntities, initPhase, entityName2, attributeName2] {
	if( ((opCPPcode == "+") || (opCPPcode == "-")) // term1 and term2 must be atoms, term 1 must be a dynamic entity attribute and they must be of same type
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE &&
	(canBeStatic || termTypeAndCppCode1.first[0] == DYNAMIC_ETYPE) && attributeName1 != ""
	&& (termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2]
	|| ( termTypeAndCppCode2.first[2] == NULL_ETYPE && (termTypeAndCppCode1.first[2] != NUMBER_ETYPE &&
	termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE)
	)
	) ) ) {
		throw HATPParserException(ERROR, nbLine, "this effect must be done between a dynamic atom and a value of same kind");
	}
	if( ((opCPPcode == "/") || (opCPPcode == "*")) // term1 and term2 must be atoms, term 1 must be a dynamic entity attribute and they must be of same type
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE &&
	(canBeStatic || termTypeAndCppCode1.first[0] == DYNAMIC_ETYPE) && attributeName1 != "" && termTypeAndCppCode2.first[2] == NUMBER_ETYPE
	&& termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2] ) ) {
		throw HATPParserException(ERROR, nbLine, "this effect must be done between a dynamic atom and a value of type number");
	}
	// everything is OK
	std::string codeWithoutReport, codeWithReport;
	// for types number, string and bool
	if(termTypeAndCppCode1.first[2] == NUMBER_ETYPE || termTypeAndCppCode1.first[2] == STRING_ETYPE || termTypeAndCppCode1.first[2] == BOOL_ETYPE) {
		codeWithoutReport = termTypeAndCppCode1.second+" = "+termTypeAndCppCode1.second + " "+opCPPcode+" "+termTypeAndCppCode2.second+";";
		codeWithReport = "report.";
		codeWithReport += "addAtomModifChange("+entityName1+"->getName(), \""+attributeName1+"\", "+
			termTypeAndCppCode1.second+", "+termTypeAndCppCode1.second + " "+opCPPcode+" " +termTypeAndCppCode2.second+");\n";
		codeWithReport += codeWithoutReport;
	}
	codeWithAndWithoutReport = std::make_pair(codeWithReport, codeWithoutReport);
} ;
//******** end effectCallTerm rule ********


//******** start preconds rule ********
//syntax: <sets of precondTerms> or OR{<sets of precondTerms>}{<sets of precondTerms>}...
// availableEntities contains names of available entities associated to their type and CPP code
preconds[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase]
	returns [std::pair<std::string, std::string> codeWithAndWithoutReports] throws HATPParserException {
	std::string code1, code2;
	std::pair<std::string, std::string> tmp;
} :
//alternative 1
(tmp=precondTerm[availableEntities, initPhase] {
	code1 += tmp.first + " && ";
	code2 += tmp.second + " && ";
}
)* {
	code1 += "true";
	code2 += "true";
	codeWithAndWithoutReports = std::make_pair(code1, code2);
}
// alternative 2
| OR LEFT_SCOPE {
	code1 += "(";
	code2 += "(";
}
(tmp=precondTerm[availableEntities, initPhase] {
	code1 += tmp.first + " && ";
	code2 += tmp.second + " && ";
}
)* RIGHT_SCOPE {
	code2 += "true) || ("; // a && true = a
	code1 += "true) || (report.clearTests() && "; // a && true = a
}
(LEFT_SCOPE (tmp=precondTerm[availableEntities, initPhase] {
	code1 += tmp.first + " && ";
	code2 += tmp.second + " && ";
}
)* RIGHT_SCOPE {
	code2 += "true) || ("; // a && true = a
	code1 += "true) || (report.clearTests() && "; // a && true = a
}
)+ {
	code1 += "false)"; // a || false = a
	code2 += "false)"; // a || false = a
	codeWithAndWithoutReports = std::make_pair(code1, code2);
} ;
//********* end preconds rule *********


//******** start precondTerm rule ********
// availableEntities contains names of available entities associated to their type and CPP code
precondTerm[std::map<std::string, std::pair<std::string, std::string> > availableEntities, bool initPhase]
returns [std::pair<std::string, std::string> codeWithAndWithoutReport] throws HATPParserException :
{
	std::pair<std::vector<std::string>, std::string> termTypeAndCppCode1, termTypeAndCppCode2;
	std::string opCPPcode;
	unsigned int nbLine;
	std::string entityName1, attributeName1, entityName2, attributeName2;
}
termTypeAndCppCode1=singleTerm[availableEntities, initPhase, entityName1, attributeName1]
( op1:TEST_EQUAL { opCPPcode="=="; nbLine=op1->getLine(); }
| op2:TEST_DIFF { opCPPcode="!="; nbLine=op2->getLine(); }
| op3:TEST_SET_IN { opCPPcode=">>"; nbLine=op3->getLine(); }
| op4:TEST_SET_NOT_IN { opCPPcode="^="; nbLine=op4->getLine(); }
| op5:SUP { opCPPcode=">"; nbLine=op5->getLine(); }
| op6:SUP_EQUAL { opCPPcode=">="; nbLine=op6->getLine(); }
| op7:INF { opCPPcode="<"; nbLine=op7->getLine(); }
| op8:INF_EQUAL { opCPPcode="<="; nbLine=op8->getLine(); }
)
termTypeAndCppCode2=singleTerm[availableEntities, initPhase, entityName2, attributeName2]
SEMICOLON
{
	// the two terms must be of the same kind
	if( !( termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2] || (termTypeAndCppCode1.first[2] == NULL_ETYPE &&
	termTypeAndCppCode2.first[2] != NUMBER_ETYPE && termTypeAndCppCode2.first[2] != STRING_ETYPE && termTypeAndCppCode2.first[2] != BOOL_ETYPE ) ||
	(termTypeAndCppCode2.first[2] == NULL_ETYPE && termTypeAndCppCode1.first[2] != NUMBER_ETYPE
	&& termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE ) ) )
	{
		throw HATPParserException(ERROR, nbLine, "comparison must be done between terms of same kind");
	}
	if( (opCPPcode == "==" || opCPPcode == "!=") // term1 and term2 must be atoms
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE ) ) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between an atoms");
	}
	if( (opCPPcode == ">>" || opCPPcode == "^=") // term1 must be an atom and term2 must be a set
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == SET_ETYPE ) ) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between an atom and a set");
	}
	if( (opCPPcode == ">" || opCPPcode == ">=" || opCPPcode == "<" || opCPPcode == "<=" )
	// term1 and term2 must be atoms and not bool
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE
	&& termTypeAndCppCode1.first[2] == NUMBER_ETYPE && termTypeAndCppCode2.first[2] == NUMBER_ETYPE )) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between not bool atoms");
	}
	// everything is OK
	std::string CPPval1 = termTypeAndCppCode1.second;
	std::string CPPval2 = termTypeAndCppCode2.second;
	std::string codeWithoutReport = "(("+CPPval1+") "+opCPPcode+" ("+CPPval2+"))";
	std::string codeWithReport = "(("+CPPval1+" "+opCPPcode+" "+CPPval2+ ")";
	bool testDiff;

	if((opCPPcode == "==") | (opCPPcode == "!=") | (opCPPcode == ">>") | (opCPPcode == "^=")) {
		if(attributeName1 != "" && opCPPcode != ">>" && opCPPcode != "^=") { // first term is obtained from an entity and an attribute
			testDiff = false;
			codeWithReport += " && report.";
			if( opCPPcode == "==" ) {
				codeWithReport += "addAtomTestEqual";
			} else if( opCPPcode == ">>" ) {
				codeWithReport += "addVectorTestIn";
			} else if( opCPPcode == "^=" ) {
				codeWithReport += "addVectorTestNotIn";
			} else if( opCPPcode == "!=" ) {
				codeWithReport += "addAtomTestDiff"; testDiff = true;
			}
			if(!testDiff) {
				if(termTypeAndCppCode2.first[2] != NUMBER_ETYPE && termTypeAndCppCode2.first[2] != STRING_ETYPE &&
				termTypeAndCppCode2.first[2] != BOOL_ETYPE && termTypeAndCppCode2.first[2] != NULL_ETYPE) {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", ("+CPPval2 +")?("+CPPval2+"->getName()):string(\"NULL\"))";
				} else if(termTypeAndCppCode2.first[2] == NULL_ETYPE) {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", "+"string(\"NULL\")"+")";
				} else {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", "+CPPval2+")";
				}
			} else { // we store in the report the current value of entity1.attribute1 to maintain precond at true
				if(termTypeAndCppCode1.first[2] != NUMBER_ETYPE && termTypeAndCppCode1.first[2] != STRING_ETYPE &&
				termTypeAndCppCode1.first[2] != BOOL_ETYPE && termTypeAndCppCode1.first[2] != NULL_ETYPE) {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", ("+CPPval1 +")?("+CPPval1+"->getName()):string(\"NULL\"))";
				} else if(termTypeAndCppCode1.first[2] == NULL_ETYPE) {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", "+"string(\"NULL\")"+")";
				} else {
					codeWithReport += "("+entityName1+"->getName(), \""+attributeName1+"\", "+CPPval1+")";
				}
			}
		}

		if(attributeName2 != "") { // second term is obtained from an entity and an attribute
			testDiff = false;
			codeWithReport += " && report.";
			if( opCPPcode == "==" ) {
				codeWithReport += "addAtomTestEqual";
			} else if( opCPPcode == ">>" ) {
				codeWithReport += "addVectorTestIn";
			} else if( opCPPcode == "^=" ) {
				codeWithReport += "addVectorTestNotIn";
			} else if( opCPPcode == "!=" ) {
				codeWithReport += "addAtomTestDiff";
				testDiff = true;
			}

			if(!testDiff) {
				if(termTypeAndCppCode1.first[2] != NUMBER_ETYPE && termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE) {
					codeWithReport += "("+entityName2+"->getName(), \""+attributeName2+"\", ("+CPPval1+")?("+CPPval1+"->getName()):string(\"NULL\"))";
				} else {
					codeWithReport += "("+entityName2+"->getName(), \""+attributeName2+"\", "+CPPval1+")";
				}
			} else { // we store in the report the current value of entity2.attribute2 to maintain precond at true
				if(termTypeAndCppCode2.first[2] != NUMBER_ETYPE && termTypeAndCppCode2.first[2] != STRING_ETYPE && termTypeAndCppCode2.first[2] != BOOL_ETYPE) {
					codeWithReport += "("+entityName2+"->getName(), \""+attributeName2+"\", ("+CPPval2+")?("+CPPval2+"->getName()):string(\"NULL\"))";
				} else {
					codeWithReport += "("+entityName2+"->getName(), \""+attributeName2+"\", "+CPPval2+")";
				}
			}
		}

	}
	codeWithReport += ")"; // TODO later: make modifications on report to consider >, <, >= and <=
	codeWithAndWithoutReport = std::make_pair(codeWithReport, codeWithoutReport);
} // END SEMICOLON for standard case

// alternative: case ForAll in precondsTerm
| FORALL LEFT_PAREN type:ENTITY_ID name: ENTITY_ID COMMA {
	std::pair<std::string, std::string> contentFORALL = std::make_pair("", "");
	std::pair<std::string, std::string> tmpPair;
	std::ostringstream st;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities2;
	if(useFORALLinPreconds) {
		throw HATPParserException(ERROR, name->getLine(), "Imbricated FORALL in preconditions is not allowed.");
	} else {
		useFORALLinPreconds = true;
	}
	if(contentFORALLinPreconds.find("PFA_"+name->getText()) != contentFORALLinPreconds.end()) {
		throw HATPParserException(ERROR, name->getLine(), "substitution entity " + name->getText() + " in ForAll already used. Conflict.");
	}
	st << "bool PFA_" << name->getText() << " = true;\n";
	st << "vector<LightEntity*> " << name->getText() << " = wb.findByType(\"" << type->getText() << "\");\n";
	st << "for(unsigned int " << name->getText() << "_iterator=0 ; " << name->getText() << "_iterator < " << name->getText() <<".size() ;";
	st << "++" << name->getText() << "_iterator) {\n";
	contentFORALL.first += st.str();
	contentFORALL.second += st.str();
	std::map<std::string, std::pair<std::string, std::string> >::const_iterator it;
	for(it = availableEntities.begin() ; it != availableEntities.end() ; it++) {
		availableEntities2[it->first] = it->second;
	}
	availableEntities2[name->getText()]=std::make_pair(type->getText(), name->getText());
	std::string newEnt;
	newEnt = name->getText()+"["+name->getText()+"_iterator]";
}
// select preconds
LEFT_SCOPE tmpPair=preconds[availableEntities2, initPhase] RIGHT_SCOPE COMMA {
	boost::replace_all_regex(tmpPair.first, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	contentFORALL.first += "if("+ tmpPair.first + "){\n";
	contentFORALL.second += "if("+ tmpPair.second +"){\n";
}
// real preconds
LEFT_SCOPE tmpPair=preconds[availableEntities2, initPhase] RIGHT_SCOPE {
	boost::replace_all_regex(tmpPair.first, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name->getText()+"(\\W)"), newEnt+"\\1");
	contentFORALL.first += "if(!("+ tmpPair.first + ")){\n PFA_" + name->getText() + " = false;" + "\n}\n}\n";
	contentFORALL.second += "if(!("+ tmpPair.second +")){\n PFA_" + name->getText() + " = false;" + "\n}\n}\n";
}
RIGHT_PAREN SEMICOLON {
	contentFORALL.first += "}\n";
	contentFORALL.second += "}\n";
	contentFORALLinPreconds["PFA_" + name->getText()] = contentFORALL;
	codeWithAndWithoutReport = std::make_pair("PFA_" + name->getText(), "PFA_" + name->getText());
	useFORALLinPreconds = false;
}
// end alternative ForAll
// alternative: case Exist in precondsTerm (opp of ForAll --> if one true then true else false)
| EXIST LEFT_PAREN type2:ENTITY_ID name2: ENTITY_ID COMMA {
	std::pair<std::string, std::string> contentFORALL = std::make_pair("", "");
	std::pair<std::string, std::string> tmpPair;
	std::ostringstream st;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities2;
	if(useFORALLinPreconds) {
		throw HATPParserException(ERROR, name2->getLine(), "Imbricated FORALL in preconditions is not allowed.");
	} else {
		useFORALLinPreconds = true;
	}
	if(contentFORALLinPreconds.find("PFA_"+name2->getText()) != contentFORALLinPreconds.end()) {
		throw HATPParserException(ERROR, name2->getLine(), "substitution entity " + name2->getText() + " in ForAll already used. Conflict.");
	}
	st << "bool PFA_" << name2->getText() << " = false;\n";
	st << "vector<LightEntity*> " << name2->getText() << " = wb.findByType(\"" << type2->getText() << "\");\n";
	st << "for(unsigned int " << name2->getText() << "_iterator=0 ; " << name2->getText() << "_iterator < " << name2->getText() <<".size() ;";
	st << "++" << name2->getText() << "_iterator) {\n";
	contentFORALL.first += st.str();
	contentFORALL.second += st.str();
	std::map<std::string, std::pair<std::string, std::string> >::const_iterator it;
	for(it = availableEntities.begin() ; it != availableEntities.end() ; it++) {
		availableEntities2[it->first] = it->second;
	}
	availableEntities2[name2->getText()]=std::make_pair(type2->getText(), name2->getText());
	std::string newEnt;
	newEnt = name2->getText()+"["+name2->getText()+"_iterator]";
}
// select preconds
LEFT_SCOPE tmpPair=preconds[availableEntities2, initPhase] RIGHT_SCOPE COMMA {
	boost::replace_all_regex(tmpPair.first, boost::regex(name2->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name2->getText()+"(\\W)"), newEnt+"\\1");
	contentFORALL.first += "if("+ tmpPair.first + "){\n";
	contentFORALL.second += "if("+ tmpPair.second +"){\n";
}
// real preconds
LEFT_SCOPE tmpPair=preconds[availableEntities2, initPhase] RIGHT_SCOPE {
	boost::replace_all_regex(tmpPair.first, boost::regex(name2->getText()+"(\\W)"), newEnt+"\\1");
	boost::replace_all_regex(tmpPair.second, boost::regex(name2->getText()+"(\\W)"), newEnt+"\\1");
	contentFORALL.first += "if("+ tmpPair.first + "){\n PFA_" + name2->getText() + " = true;" + "\n}\n}\n";
	contentFORALL.second += "if("+ tmpPair.second +"){\n PFA_" + name2->getText() + " = true;" + "\n}\n}\n";
}
RIGHT_PAREN SEMICOLON {
	contentFORALL.first += "}\n";
	contentFORALL.second += "}\n";
	contentFORALLinPreconds["PFA_" + name2->getText()] = contentFORALL;
	codeWithAndWithoutReport = std::make_pair("PFA_" + name2->getText(), "PFA_" + name2->getText());
	useFORALLinPreconds = false;
}
// end alternative EXIST
;

//******** end precondTerm rule ********

//******** start htnDef rule ********
//syntax: HTN {....}
htnDef throws HATPParserException :
HTN LEFT_SCOPE (action | method | beliefManagement | commAction)* RIGHT_SCOPE {
	std::ostringstream st;
	std::vector<std::string> argsInDecomposition, normalArgs;
	unsigned int i;
	//we explore tasks_in_decompositions to ensure that all its members are in htnTasks
	std::map<int, std::pair<std::string, std::vector<std::string> > >::const_iterator it;
	for(it = tasks_in_decompositions.begin() ; it != tasks_in_decompositions.end() ; ++it) {
		if(htnTasks.find((it->second).first) == htnTasks.end()) {// the task has not been found
			throw HATPParserException(ERROR, it->first, "task " + (it->second).first + " unknown");
		}
		normalArgs = htnTasks.find((it->second).first)->second;
		argsInDecomposition = (it->second).second;
		if(normalArgs.size() != argsInDecomposition.size()) {// too less or too much arguments
			throw HATPParserException(ERROR, it->first, "Too much or too less arguments for task " + (it->second).first);
		}
		for(i=0 ; i<argsInDecomposition.size() ; ++i) {
			if( !argsInDecomposition[i].empty() && argsInDecomposition[i] != normalArgs[i] ) {
				st.str("");
				st << (i+1);
				throw HATPParserException(ERROR, it->first, "Type of argument "+ st.str() +" of task "+ (it->second).first +" must be "+ normalArgs[i] +" and not "+ argsInDecomposition[i]);
			}
		}
	}
	// everything OK
	std::cout << "HTN description done." << std::endl;
} ;
//********* end htnDef rule *********

//******** start action rule ********
//syntax: action <action name>(<list of parameters>) {preconditions{...}; effects{...}; cost{...};}
//<action name> must be unique
action throws HATPParserException : {
	current_task_name.clear();
	current_task_parameters.clear();
	current_task_select_terms.clear();
	std::string tmp, CPPprecond, CPPprecondWithReports, CPPeffect, CPPeffectWithReports, CPPcost, CPPtime,
		CPPparameters, CPPConstparameters, CPPparametersWithReports, CPPConstparametersWithReports;
	std::pair<std::string, std::string> tmpPair, eTypeAndCPPcode;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	std::pair<std::vector<std::string>, std::string> argTypeAndCppCode;
	std::pair<std::string, std::string> ConstAndNonConstcode;
	std::string tmp1, tmp2;
}
ACTION actionName:ENTITY_ID {
	current_task_name = actionName->getText();
	if(htnTasks.find(current_task_name) != htnTasks.end()) {// there is already an action with the same name
		throw HATPParserException(ERROR, actionName->getLine(), "task " + current_task_name + " already exists");
	}
	std::vector<std::string> vec;
	htnTasks[current_task_name] = vec;
	htnTasksIsAction[current_task_name] = true;
}
LEFT_PAREN ( (ConstAndNonConstcode=parameter COMMA {
	CPPConstparameters += ConstAndNonConstcode.first + ";\n";
	CPPparameters += ConstAndNonConstcode.second + ";\n";
	CPPConstparametersWithReports += ConstAndNonConstcode.first + ";\n";
	CPPparametersWithReports += ConstAndNonConstcode.second + ";\n";
}
)* ConstAndNonConstcode=parameter {
	CPPConstparameters += ConstAndNonConstcode.first + ";\n";
	CPPparameters += ConstAndNonConstcode.second + ";\n";
	CPPConstparametersWithReports += ConstAndNonConstcode.first + ";\n";
	CPPparametersWithReports += ConstAndNonConstcode.second + ";\n";
}
)? RIGHT_PAREN LEFT_SCOPE PRECONDITIONS
{
	this->completeMapWithTaskParameters(availableEntities);
	contentFORALLinPreconds.clear();
	useFORALLinPreconds = false;
	std::map<std::string, std::pair<std::string, std::string > >::iterator itFORALLinPreconds;
}
LEFT_SCOPE tmpPair=preconds[availableEntities, false] {
	CPPprecond = "bool "+ current_task_name +"Precond(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n" + CPPConstparameters;
	for(itFORALLinPreconds = contentFORALLinPreconds.begin() ; itFORALLinPreconds != contentFORALLinPreconds.end() ; itFORALLinPreconds++) {
		CPPprecond += itFORALLinPreconds->second.second;
	}
	CPPprecond += "\nif(" + tmpPair.second + ")\n return true;\n\nreturn false;\n}";
	CPPprecondWithReports = "bool "+ current_task_name
	+"PrecondWithReports(LightWorldBase const& wb, vector<LightEntity const*> const& args, WorldBaseReport& report)\n{\n" + CPPConstparametersWithReports;
	for(itFORALLinPreconds = contentFORALLinPreconds.begin() ; itFORALLinPreconds != contentFORALLinPreconds.end() ; itFORALLinPreconds++) {
		CPPprecondWithReports += itFORALLinPreconds->second.first;
	}
	CPPprecondWithReports += "\nif(" + tmpPair.first + ")\n return true;\n\nreturn false;\n}";
}
RIGHT_SCOPE SEMICOLON EFFECTS LEFT_SCOPE tmpPair=effects[availableEntities, false, false] {
	CPPeffect = "void "+ current_task_name +"Effects(LightWorldBase& wb, vector<LightEntity*> const& args)\n{\n"+ CPPparameters +"\n"+ tmpPair.second +"}\n";
	CPPeffectWithReports = "void "+ current_task_name +"EffectsWithReports(LightWorldBase& wb, vector<LightEntity*> const& args, WorldBaseReport& report)\n{\n"+ CPPparametersWithReports +"\n"+ tmpPair.first +"}\n";
}
RIGHT_SCOPE SEMICOLON
COST LEFT_SCOPE costFctName:OTHER_ID LEFT_PAREN
{
	CPPcost = "float "+ current_task_name +"Cost(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"+ CPPConstparameters +"\nreturn "+ costFctName->getText() +"(";
}
( argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCppCode.first)[2] == NUMBER_ETYPE) {
		CPPcost += argTypeAndCppCode.second;
	} else if((argTypeAndCppCode.first)[2] == STRING_ETYPE) {
		CPPcost += "\""+argTypeAndCppCode.second+"\"";
	} else {
		CPPcost += "(*"+argTypeAndCppCode.second+")";
	}
}
( COMMA argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCppCode.first)[2] == NUMBER_ETYPE) {
		CPPcost += ", "+argTypeAndCppCode.second;
	} else if((argTypeAndCppCode.first)[2] == STRING_ETYPE) {
		CPPcost += ", \""+argTypeAndCppCode.second+"\"";
	} else {
		CPPcost += ", (*"+argTypeAndCppCode.second+")";
	}
}
)*
)?
RIGHT_PAREN { CPPcost += ");\n}"; }
RIGHT_SCOPE SEMICOLON
DURATION LEFT_SCOPE timeFctName:OTHER_ID LEFT_PAREN
{
	CPPtime = "pair<double,double> "+ current_task_name +"Time(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"+ CPPConstparameters +"\nreturn "+ timeFctName->getText() +"(";
}
( argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCppCode.first)[2] == NUMBER_ETYPE) {
		CPPtime += argTypeAndCppCode.second;
	} else if((argTypeAndCppCode.first)[2] == STRING_ETYPE) {
		CPPtime += "\""+argTypeAndCppCode.second+"\"";
	} else {
		CPPtime += "(*"+argTypeAndCppCode.second+")";
	}
}
( COMMA argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCppCode.first)[2] == NUMBER_ETYPE) {
		CPPtime += ", "+argTypeAndCppCode.second;
	} else if((argTypeAndCppCode.first)[2] == STRING_ETYPE) {
		CPPtime += ", \""+argTypeAndCppCode.second+"\"";
	} else {
		CPPtime += ", (*"+argTypeAndCppCode.second+")";
	}
}
)*
)?
RIGHT_PAREN { CPPtime += ");\n}"; }
RIGHT_SCOPE SEMICOLON
RIGHT_SCOPE {
	// we compute the code associated to the action and we store it in htnTasksCode
	std::string actionCPPCode = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n"+ CPPprecond +"\n\n"+
	CPPprecondWithReports+"\n\n"+ CPPeffect +"\n\n"+ CPPeffectWithReports +"\n\n"+ CPPcost+
	"\n\n"+ CPPtime +"\n";
	htnTasksCode[current_task_name] = actionCPPCode;
} ;
//********* end action rule *********

//******** start method rule ********
//syntax: method <action name>(<list of parameters>) {preconditions{...}; effects{...}; cost{...};}
//<action name> must be unique
method throws HATPParserException : {
	current_task_name.clear();
	current_task_parameters.clear();
	current_task_select_terms.clear();
	std::pair<std::string, std::string> tmp1;
	std::string tmp, tmp2, CPPempty, CPPdecompo , CPPConstParameters, CPPparameters;
	std::pair<std::string, std::string> ConstAndNonConstcode;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
}
METHOD methodName:ENTITY_ID {
	current_task_name = methodName->getText();
	if(htnTasks.find(current_task_name) != htnTasks.end()) {// there is already an action with the same name
		throw HATPParserException(ERROR, methodName->getLine(), "task " + current_task_name + " already exists");
	}
	std::vector<std::string> vec;
	htnTasks[current_task_name] = vec;
	htnTasksIsAction[current_task_name] = false;
	unsigned int nbDecompo=0;
}
LEFT_PAREN
( (ConstAndNonConstcode=parameter COMMA {
	CPPConstParameters += ConstAndNonConstcode.first + ";\n";
	CPPparameters += ConstAndNonConstcode.second + ";\n";
}
)* ConstAndNonConstcode=parameter {
	CPPConstParameters += ConstAndNonConstcode.first + ";\n";
	CPPparameters += ConstAndNonConstcode.second + ";\n";
}
)?
RIGHT_PAREN {
	this->completeMapWithTaskParameters(availableEntities);
	contentFORALLinPreconds.clear();
	useFORALLinPreconds = false;
	std::map<std::string, std::pair<std::string, std::string > >::iterator itFORALLinPreconds;
}
LEFT_SCOPE EMPTY LEFT_SCOPE tmp1=preconds[availableEntities, false] {
	CPPempty = "bool "+ current_task_name +"Empty(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"+ CPPConstParameters;
	for(itFORALLinPreconds = contentFORALLinPreconds.begin() ; itFORALLinPreconds != contentFORALLinPreconds.end() ; itFORALLinPreconds++) {
		CPPempty += itFORALLinPreconds->second.second;
	}
	CPPempty += "\nif(" + tmp1.second + ")\n return true;\n\nreturn false;\n}";
}
RIGHT_SCOPE SEMICOLON {
	CPPdecompo = "vector<TaskList*> "+ current_task_name +"Decompo(LightWorldBase const& wb, HTNBase const& taskBase, vector<LightEntity const*> const& args)\n{\n"+ CPPConstParameters +"\n";
	CPPdecompo += "vector<TaskList*> vec;\nvector<string> tmp_args;\n(void) tmp_args;\n";
	CPPdecompo += "map<unsigned int, TaskInstance*> map_tasks;\nmap<unsigned int, vector<unsigned int> > map_links;\n\n";
}
( {
	nbDecompo++;
	contentFORALLinPreconds.clear();
	useFORALLinPreconds = false;
	std::map<std::string, std::pair<std::string, std::string > >::iterator itFORALLinPreconds;
}
LEFT_SCOPE PRECONDITIONS LEFT_SCOPE tmp1=preconds[availableEntities, false] RIGHT_SCOPE SEMICOLON SUBTASKS LEFT_SCOPE tmp2=decomposition[nbDecompo] {
	std::ostringstream st;
	st << nbDecompo;
	CPPdecompo += "// task "+ current_task_name +": decomposition " + st.str() +"\n";
	for(itFORALLinPreconds = contentFORALLinPreconds.begin() ; itFORALLinPreconds != contentFORALLinPreconds.end() ; itFORALLinPreconds++) {
		CPPdecompo += itFORALLinPreconds->second.second;
	}
	CPPdecompo += "\nif(" + tmp1.second + ")\n{\n" + tmp2 + "}\n\n";
}
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE )+ {
	CPPdecompo += "return vec;\n}";
	// we compute the code associated to the method and we store it in htnTasksCode
	std::string methodCPPCode = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n" + CPPempty + "\n\n" + CPPdecompo + "\n";
	htnTasksCode[current_task_name] = methodCPPCode;
}
RIGHT_SCOPE ;
//********* end method rule *********

//******** start parameter rule ********
//syntax: <type> <name> //type must be a defined entity type
parameter returns[std::pair<std::string, std::string> ConstAndNonConstcode] throws HATPParserException :
	type:ENTITY_ID name:ENTITY_ID {
	if(definedEntityTypes.find(type->getText()) == definedEntityTypes.end()) {// type is unknown
		throw HATPParserException(ERROR, type->getLine(), "type " + type->getText() + " unknown");
	}
	if(current_task_parameters.find(name->getText()) != current_task_parameters.end()) {// parameter name has already been declared
		throw HATPParserException(ERROR, name->getLine(), "parameter name " + name->getText() + " used previously in the task");
	}
	// everything OK, we update useful data
	htnTasks[current_task_name].push_back(type->getText());
	current_task_parameters[name->getText()]=type->getText();
	std::ostringstream st1, st2;
	// attribute "unused" necessary to prevent warning message "unused variable"
	st1 << "LightEntity const* ";
	st2 << "LightEntity* ";
	st1 << name->getText() << " = args[" << current_task_parameters.size()-1 << "];\n(void)"+name->getText();
	st2 << name->getText() << " = args[" << current_task_parameters.size()-1 << "];\n(void)"+name->getText();
	ConstAndNonConstcode = std::make_pair(st1.str(), st2.str());
} ;
//********* end parameter rule *********

//******** start decomposition rule ********
//a set of selectTerms and a set of tasks with causal links between them
// argument nbDecomposition is used to ensure that C++ variables have unique names
decomposition[int nbDecomposition] returns[std::string code] throws HATPParserException : {
	current_task_select_terms.clear();
	current_task_select_once.clear();
	current_task_select_terms_code.clear();
	unsigned int i;
	std::string tmp, CPPloopsCode;
	std::ostringstream st;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
}
(tmp=selectTerm[nbDecomposition] { CPPloopsCode += tmp; }
)* {
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		st.str(""); // we clear st
		st << "vector<LightEntity*> " << current_task_select_terms[i].first << "_" << nbDecomposition; // variable name <select term name>_<nbDecomposition>
		st << " = wb.findByType(\"" << current_task_select_terms[i].second << "\");\n";
		code += st.str();
	}
	code += CPPloopsCode;
	code += "map_tasks.clear();\nmap_links.clear();\n";
	this->completeMapWithTaskParameters(availableEntities);
	this->completeMapWithTaskSelectTerms(availableEntities);
}
tmp=taskListDescription[nbDecomposition, availableEntities] {
	code += tmp;
	code += "vec.push_back(new TaskList(map_tasks, map_links));\n";
	for(i=current_task_select_terms.size() ; i > 0 ; --i) {
		if(current_task_select_once[i-1] == true) {
			code += "break;\n";
		}
		code += "}\n"; // close select precond if
		code += "}\n"; // close select for loop
	}
	// decomposition description is finished: we reset select terms
	current_task_select_terms.clear();
	current_task_select_once.clear();
	current_task_select_terms_code.clear();
};
//********* end decomposition rule *********

//******** start selectTerm rule ********
// argument nbDecomposition is used to ensure that C++ variables have unique names
//syntax: <name> = SELECT/SELECTONCE(<type>, {<preconds>});
selectTerm[int nbDecomposition] returns[std::string code] throws HATPParserException : {
	unsigned int i;
	std::ostringstream st;
	std::string CPPvarName, CPPiteratorName, tmp, tmp1, tmp2;
	std::pair<std::string, std::string> argTypeAndCPPcode;
	std::pair<std::vector<std::string>, std::string> argTypeAndCPPcode2;
	std::pair<std::string, std::string> tmpPair;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	bool isSelectOnce = false;
}
	name:ENTITY_ID EQUAL
((
(SELECT {isSelectOnce = false;} | SELECTONCE {isSelectOnce = true;})
LEFT_PAREN typeName:ENTITY_ID {
	// typeName must be a known type and name must not exist in current_task_parameters and current_task_select_terms
	code = "";
	if( definedEntityTypes.find(typeName->getText()) == definedEntityTypes.end() ) {// type is unknown
		throw HATPParserException(ERROR, typeName->getLine(), "type " + typeName->getText() + " unknown");
	}
	if( current_task_parameters.find(name->getText()) != current_task_parameters.end()) {
		throw HATPParserException(ERROR, name->getLine(), "selected entity " + name->getText() + " has the same name as a task parameter");
	}
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		if( current_task_select_terms[i].first == name->getText()) {
			throw HATPParserException(ERROR, name->getLine(), "selected entity " + name->getText() + " has the same name as a previous selected entity");
		}
	}
	//everything OK, we add name with type typeName in current_task_select_terms
	current_task_select_terms.push_back( std::make_pair(name->getText(), typeName->getText()) );
	current_task_select_once.push_back( isSelectOnce );
	// iterator for "for" loops will have the name <select term name>_<nbDecomposition>_iterator
	//we complete C++ code
	st << current_task_select_terms[i].first << "_" << nbDecomposition;
	CPPvarName = st.str();
	CPPiteratorName = CPPvarName + "_iterator";
	code += "for(unsigned int "+ CPPiteratorName +"=0 ; "+ CPPiteratorName +"<"+ CPPvarName + ".size() ; ++"+ CPPiteratorName +")\n{\n";
	// we complete current_task_select_terms_code
	current_task_select_terms_code.push_back( "(" + CPPvarName +"["+ CPPiteratorName +"])" );
	this->completeMapWithTaskParameters(availableEntities);
	this->completeMapWithTaskSelectTerms(availableEntities);
}
COMMA LEFT_SCOPE tmpPair=preconds[availableEntities, false] {
	code += "if("+ tmpPair.second +")\n{\n"; }
RIGHT_SCOPE RIGHT_PAREN SEMICOLON
)|(
SELECTORDERED LEFT_PAREN typeName2:ENTITY_ID {
	isSelectOnce = false;
	code = "";
	if( definedEntityTypes.find(typeName2->getText()) == definedEntityTypes.end() ) {// type is unknown
		throw HATPParserException(ERROR, typeName2->getLine(), "type " + typeName2->getText() + " unknown");
	}
	if( current_task_parameters.find(name->getText()) != current_task_parameters.end()) {
		throw HATPParserException(ERROR, name->getLine(), "selected entity " + name->getText() + " has the same name as a task parameter");
	}
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		if( current_task_select_terms[i].first == name->getText()) {
			throw HATPParserException(ERROR, name->getLine(), "selected entity " + name->getText() + " has the same name as a previous selected entity");
		}
	}
	current_task_select_terms.push_back( std::make_pair(name->getText(), typeName2->getText()) );
	current_task_select_once.push_back( isSelectOnce );
	// iterator for "for" loops will have the name <select term name>_<nbDecomposition>_iterator
	//we complete C++ code
	st << current_task_select_terms[i].first << "_" << nbDecomposition;
	CPPvarName = st.str();
	CPPiteratorName = CPPvarName + "_iterator";
	code += "map<double, vector<LightEntity*> > " + CPPvarName +"Temp;\n";
	code += "for(unsigned int "+ CPPiteratorName +"=0 ; "+ CPPiteratorName +"<"+ CPPvarName + ".size() ; ++"+ CPPiteratorName +")\n{\n";
	// we complete current_task_select_terms_code
	current_task_select_terms_code.push_back( "(" + CPPvarName +"["+ CPPiteratorName +"])" );
	this->completeMapWithTaskParameters(availableEntities);
	this->completeMapWithTaskSelectTerms(availableEntities);
}
COMMA leftp:LEFT_SCOPE tmpPair=preconds[availableEntities, false]
RIGHT_SCOPE COMMA typeFunc2:OTHER_ID LEFT_PAREN {
	code += "if("+ tmpPair.second +")\n{\n";
	code += "double " + CPPvarName +"ValTemp = " + typeFunc2->getText() + "(";
}
( argTypeAndCPPcode2=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCPPcode2.first)[2] == NUMBER_ETYPE) {
		code += argTypeAndCPPcode2.second;
	} else if((argTypeAndCPPcode2.first)[2] == STRING_ETYPE) {
		code += "\""+argTypeAndCPPcode2.second+"\"";
	} else {
		code += "(*"+argTypeAndCPPcode2.second+")";
	}
}
( COMMA argTypeAndCPPcode2=singleTerm[availableEntities,false,tmp1,tmp2]
{
	if((argTypeAndCPPcode2.first)[2] == NUMBER_ETYPE) {
		code += ", "+argTypeAndCPPcode2.second;
	} else if((argTypeAndCPPcode2.first)[2] == STRING_ETYPE) {
		code += ", \""+argTypeAndCPPcode2.second+"\"";
	} else {
		code += ", (*"+argTypeAndCPPcode2.second+")";
	}
}
)*
)?
RIGHT_PAREN {
	code += ");\n";
	code += "(" + CPPvarName +"Temp[" + CPPvarName +"ValTemp]).push_back(" + CPPvarName +"["+ CPPiteratorName +"]);\n";
	code += "}}\n";
	code += CPPvarName +".clear();\n";
}
COMMA (INF {
	code += "for(map<double, vector<LightEntity*> >::iterator "+CPPvarName+"Temp_iterator = "+CPPvarName+"Temp.begin(); ";
	code += CPPvarName+"Temp_iterator != "+CPPvarName+"Temp.end(); " + CPPvarName+"Temp_iterator++){\n";
}
| SUP {
	code += "for(map<double, vector<LightEntity*> >::reverse_iterator "+CPPvarName+"Temp_iterator = "+CPPvarName+"Temp.rbegin(); ";
	code += CPPvarName+"Temp_iterator != "+CPPvarName+"Temp.rend(); " + CPPvarName+"Temp_iterator++){\n";
})
{
	code += "for(unsigned int itemp = 0 ; itemp < (*"+CPPvarName+"Temp_iterator).second.size(); itemp++){\n";
	code += CPPvarName +".push_back((*"+CPPvarName+"Temp_iterator).second[itemp]);\n";
	code += "}}\n";
	code += "for(unsigned int "+ CPPiteratorName +"=0 ; "+ CPPiteratorName +"<"+ CPPvarName + ".size() ; ++"+ CPPiteratorName +")\n{\n";
	code += "if("+ tmpPair.second +")\n{\n";
}
RIGHT_PAREN SEMICOLON
)) ;
//********* end selectTerm rule *********

//******** start taskListDescription rule ********
// argument nbDecomposition is used to ensure that C++ variables have unique names, it is the number of the current decomposition
//syntax: <nb> : <task_name>(<arg1>, ..., <argn>) > <nb1>, > <nb2>, ... ;
taskListDescription[int nbDecomposition, std::map<std::string, std::pair<std::string, std::string> > availableEntities]
returns[std::string code] throws HATPParserException : {
	std::ostringstream st;
	std::string nb_string;
	std::pair<std::vector<std::string>, std::string> argsTypesAndCPPcode;
	current_decomposition_nodes.clear();
	unsigned int nb, tmp_nb;
}
(id:NUMBER_ID {
	// id must be an unsigned int, we use boost::lexical_cast
	try {
		nb = boost::lexical_cast<unsigned int>(id->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, id->getLine(), "ID of decomposition node " + id->getText() + " must be an unsigned int");
	}
	// id must have not been not declared before
	if(current_decomposition_nodes.find(nb) != current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, id->getLine(), "ID " + id->getText() + " has already been used in this decomposition");
	}
	// we do not add nb in current_decomposition_nodes now because we have to check links validity before
	// we get nb under a string form to produce C++ code
	st.str("");
	st << nb;
	nb_string = st.str();
}
COLON taskName:ENTITY_ID
{ code += "map_links["+ nb_string +"] = vector<unsigned int>(0);\ntmp_args.clear();\n"; }
	leftp:LEFT_PAREN argsTypesAndCPPcode=subtasksArguments[availableEntities,leftp->getLine()] {
	code += argsTypesAndCPPcode.second;
	// we store taskName and its arguments types (if they have been identified) to ensure HTN consistency after all tasks have been defined
	std::pair<std::string, std::vector<std::string> > args = std::make_pair(taskName->getText(), argsTypesAndCPPcode.first);
	tasks_in_decompositions.insert( std::make_pair(taskName->getLine(), args) );
}
RIGHT_PAREN { code += "map_tasks["+ nb_string +"] = new TaskInstance(\""+ taskName->getText() +"\", tmp_args, wb);\n"; }
( (SUP idNext:NUMBER_ID COMMA {
	// idNext must be an unsigned int (we use boost::lexical_cast) and must have been declared before
	try {
		tmp_nb = boost::lexical_cast<unsigned int>(idNext->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, idNext->getLine(), "ID of decomposition node " + idNext->getText() + " must be an unsigned int");
	}
	// id must have been declared before
	if(current_decomposition_nodes.find(tmp_nb) == current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, idNext->getLine(), "ID " + idNext->getText() + " unknown at this point");
	}
	code += "map_links["+ nb_string +"].push_back("+ idNext->getText() +");\n";
}
)* SUP idNextEnd:NUMBER_ID {
	// idNextEnd must be an unsigned int (we use boost::lexical_cast) and must have been declared before
	try {
		tmp_nb = boost::lexical_cast<unsigned int>(idNextEnd->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, idNextEnd->getLine(), "ID of decomposition node " + idNextEnd->getText() + " must be an unsigned int");
	}
	// id must have been declared before
	if(current_decomposition_nodes.find(tmp_nb) == current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, idNextEnd->getLine(), "ID " + idNextEnd->getText() + " unknown at this point");
	}
	code += "map_links["+ nb_string +"].push_back("+ idNextEnd->getText() +");\n";
}
)?
SEMICOLON {
	// everything is OK in this line of decomposition description, we add nb in current_decomposition_nodes
	current_decomposition_nodes.insert(nb);
	code += "\n";
}
)+ ;
//********* end taskListDescription rule *********

//******** start subtasksArguments rule ********
subtasksArguments[std::map<std::string, std::pair<std::string, std::string> > availableEntities, int line]
returns[std::pair<std::vector<std::string>, std::string> argsTypesAndCPPcode] throws HATPParserException : {
	std::string code;
	std::vector<std::string> argTypes;
	std::pair<std::string, std::string> argTypeAndCPPcode;
}
( { code += "tmp_args.push_back( "; }
argTypeAndCPPcode=subtasksOneArgument[availableEntities, line] {
	argTypes.push_back(argTypeAndCPPcode.first);
	code += argTypeAndCPPcode.second + "->getName() );\n";
}

( COMMA { code += "tmp_args.push_back( "; }
argTypeAndCPPcode=subtasksOneArgument[availableEntities, line] {
	argTypes.push_back(argTypeAndCPPcode.first);
	code += argTypeAndCPPcode.second + "->getName() );\n";
}
)*
)?
{ argsTypesAndCPPcode =std::make_pair(argTypes, code) ; } ;
//********* end subtasksArguments rule *********

//******** start subtasksOneArgument rule ********
subtasksOneArgument[std::map<std::string, std::pair<std::string, std::string> > availableEntities, int line]
returns[std::pair<std::string, std::string> argTypeAndCPPcode] throws HATPParserException : {
	std::string entityName, attributeName;
	std::pair<std::vector<std::string>, std::string> typeAndCppCode;
}
typeAndCppCode=singleTerm[availableEntities, false, entityName, attributeName] {
	// the single Term must be an atom entity
	if(typeAndCppCode.first[1] != ATOM_ETYPE || typeAndCppCode.first[2] == NUMBER_ETYPE
	|| typeAndCppCode.first[2] == STRING_ETYPE || typeAndCppCode.first[2] == BOOL_ETYPE
	|| typeAndCppCode.first[2] == NULL_ETYPE) {
		throw HATPParserException(ERROR, (unsigned int)line, "an argument must be a non null atomic Entity");
	}
	argTypeAndCPPcode = std::make_pair(typeAndCppCode.first[2], typeAndCppCode.second);
} ;
//********* end subtasksOneArgument rule *********

//******** start evalPredicateArguments rule ********
evalPredicateArguments[std::map<std::string, std::pair<std::string, std::string> > availableEntities, int line]
returns[std::pair<std::vector<std::string>, std::string> argsTypesAndCPPcode] throws HATPParserException : {
	std::string code;
	std::vector<std::string> argTypes;
	std::pair<std::string, std::string> argTypeAndCPPcode;
}
( { code += ""; }
	argTypeAndCPPcode=evalPredicateOneArgument[availableEntities, line] {
	argTypes.push_back(argTypeAndCPPcode.first);
	code += argTypeAndCPPcode.second + "";
}

( COMMA { code += ", "; }
	argTypeAndCPPcode=evalPredicateOneArgument[availableEntities, line] {
	argTypes.push_back(argTypeAndCPPcode.first);
	code += argTypeAndCPPcode.second + "";
})*)?
{ argsTypesAndCPPcode =std::make_pair(argTypes, code) ; } ;
//********* end evalPredicateArguments rule *********

//******** start evalPredicateOneArgument rule ********
evalPredicateOneArgument[std::map<std::string, std::pair<std::string, std::string> > availableEntities, int line]
returns[std::pair<std::string, std::string> argTypeAndCPPcode] throws HATPParserException : {
	std::string entityName, attributeName;
	std::pair<std::vector<std::string>, std::string> typeAndCppCode;
}
typeAndCppCode=singleTerm[availableEntities, false, entityName, attributeName] {
	// the single Term must be an atom entity
	if(typeAndCppCode.first[1] != ATOM_ETYPE
	|| typeAndCppCode.first[2] == BOOL_ETYPE
	|| typeAndCppCode.first[2] == NULL_ETYPE){
		throw HATPParserException(ERROR, (unsigned int)line, "an eval predicate argument must be a non null atomic Entity");
	}
	argTypeAndCPPcode = std::make_pair(typeAndCppCode.first[2], typeAndCppCode.second);
} ;
//********* end evalPredicateOneArgument rule *********

//******** start parameterComm rule ********
parameterComm returns[std::pair<std::string, std::string> ConstAndNonConstcode] throws HATPParserException :
	type:ENTITY_ID name:ENTITY_ID {
	if(definedEntityTypes.find(type->getText()) == definedEntityTypes.end() && type->getText() != ENTITY_ETYPE && type->getText() != ATTR_ETYPE) {
		throw HATPParserException(ERROR, type->getLine(), "type " + type->getText() + " unknown in CommAction");
	}
	if(current_task_parameters.find(name->getText()) != current_task_parameters.end()) {
		throw HATPParserException(ERROR, name->getLine(), "parameter name " + name->getText() + " used previously in the task");
	}
	current_task_parameters[name->getText()]=type->getText();
	std::ostringstream st1, st2;
	st1 << "LightEntity const* ";
	st2 << "LightEntity* ";
	st1 << name->getText() << " = args[" << current_task_parameters.size()-1 << "];\n(void)"+name->getText();
	st2 << name->getText() << " = args[" << current_task_parameters.size()-1 << "];\n(void)"+name->getText();
	ConstAndNonConstcode = std::make_pair(st1.str(), st2.str());
} ;
//********* end parameterComm rule *********

//******** start BeliefManagement rule ********
//syntax: beliefManagement { ... }
beliefManagement throws HATPParserException : {
	if(!myselfAlreadyDefined) {
		throw HATPParserException(ERROR, 0, "Cannot define beliefManagement Method: agent myself is undefined");
	}
	beliefMethodDefined = true;
	std::string cppActPreDef, cppActEffDef, cppActCostDef, cppActTimeDef;
	std::string cppEDef = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n";
}
BMANAGEMENT LEFT_SCOPE
INFORMATION LEFT_SCOPE iAction:ENTITY_ID SEMICOLON RIGHT_SCOPE {
	// Comm action for information
	commActionNameList[INFORMATION_TAG] = iAction->getText();
	cppActPreDef = "bool "+ iAction->getText() +"Precond(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActPreDef += "return true;\n}\n";
	cppActEffDef = "void "+ iAction->getText() +"Effects(LightWorldBase& wb, vector<LightEntity*> const& args)\n{\n";
	cppActEffDef += "wb.updateForAgent(args[0]->getName(), args[1]->getName(), args[2]->getName(), args[3]->getName());\n";
	cppActEffDef += "\n}\n";
	cppActCostDef = "float "+ iAction->getText() +"Cost(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActCostDef += "return cost1();";
	cppActCostDef += "\n}\n";
	cppActTimeDef = "pair<double,double> "+ iAction->getText() +"Time(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActTimeDef += "return duration1();";
	cppActTimeDef += "\n}\n";
	commActionDefaultList[iAction->getText()] = cppEDef + cppActPreDef + cppActEffDef + cppActCostDef + cppActTimeDef;
}
CONTRADICTION LEFT_SCOPE cAction:ENTITY_ID SEMICOLON RIGHT_SCOPE {
	// Comm action for contradiction
	commActionNameList[CONTRADICTION_TAG] = cAction->getText();
	cppActPreDef = "bool "+ cAction->getText() +"Precond(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActPreDef += "return true;\n}\n";
	cppActEffDef = "void "+ cAction->getText() +"Effects(LightWorldBase& wb, vector<LightEntity*> const& args)\n{\n";
	cppActEffDef += "wb.updateForAgent(args[0]->getName(), args[1]->getName(), args[2]->getName(), args[3]->getName());\n";
	cppActEffDef += "\n}\n";
	cppActCostDef = "float "+ cAction->getText() +"Cost(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActCostDef += "return cost1();";
	cppActCostDef += "\n}\n";
	cppActTimeDef = "pair<double,double> "+ cAction->getText() +"Time(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n";
	cppActTimeDef += "return duration1();";
	cppActTimeDef += "\n}\n";
	commActionDefaultList[cAction->getText()] = cppEDef + cppActPreDef + cppActEffDef + cppActCostDef + cppActTimeDef;
}
QUESTION LEFT_SCOPE qAction:ENTITY_ID SEMICOLON RIGHT_SCOPE {
	// Comm action for question
	commActionNameList[QUESTION_TAG] = qAction->getText();
	cppActPreDef = "bool "+ qAction->getText() +"Precond(LightWorldBase const& wb, vector<LightEntity const*> const& args) {\n";
	cppActPreDef += "return true;\n}\n";
	cppActEffDef = "void "+ qAction->getText() +"Effects(LightWorldBase& wb, vector<LightEntity*> const& args) {\n";
	cppActEffDef += "wb.updateForAgent(args[0]->getName(), args[1]->getName(), args[2]->getName(), args[3]->getName());\n";
	cppActEffDef += "\n}\n";
	cppActCostDef = "float "+ qAction->getText() +"Cost(LightWorldBase const& wb, vector<LightEntity const*> const& args) {\n";
	cppActCostDef += "return cost1();";
	cppActCostDef += "\n}\n";
	cppActTimeDef = "pair<double,double> "+ qAction->getText() +"Time(LightWorldBase const& wb, vector<LightEntity const*> const& args) {\n";
	cppActTimeDef += "return duration1();";
	cppActTimeDef += "\n}\n";
	commActionDefaultList[qAction->getText()] = cppEDef + cppActPreDef + cppActEffDef + cppActCostDef + cppActTimeDef;
}
RIGHT_SCOPE;

//******** start CommAction rule ********
//syntax: redefinition of Communication Actions
commAction throws HATPParserException : {
	current_task_parameters.clear();
	std::pair<std::string, std::string> tmpPair;
	std::string CPPdef, CPPparameters, CPPConstparameters, CPPprecond, CPPeffect, CPPcost, CPPtime, tmp1, tmp2;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	std::pair<std::vector<std::string>, std::string> argTypeAndCppCode;
}
COMMACTION actionName: ENTITY_ID {
	CPPdef = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n";
}
// args ...
LEFT_PAREN ( (tmpPair=parameterComm COMMA {
	CPPConstparameters += tmpPair.first + ";\n";
	CPPparameters += tmpPair.second + ";\n";
} )* tmpPair=parameterComm {
	CPPConstparameters += tmpPair.first + ";\n";
	CPPparameters += tmpPair.second + ";\n";
} )? RIGHT_PAREN LEFT_SCOPE
{this->completeMapWithTaskParameters(availableEntities);}
// preconditions ...
PRECONDITIONS LEFT_SCOPE tmpPair=preconds[availableEntities, false] {
	CPPprecond = "bool "+ actionName->getText() +"Precond(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"
	+ CPPConstparameters +"\nif(" + tmpPair.second + ")\n return true;\n\nreturn false;\n}\n\n";
} RIGHT_SCOPE SEMICOLON
// effects ...
EFFECTS LEFT_SCOPE tmpPair=effects[availableEntities, false, false] {
	CPPeffect = "void "+ actionName->getText() +"Effects(LightWorldBase& wb, vector<LightEntity*> const& args)\n{\n"+ CPPparameters +"\n"+ tmpPair.second +"}\n\n";
} RIGHT_SCOPE SEMICOLON
// cost ...
COST LEFT_SCOPE costFctName:OTHER_ID LEFT_PAREN {
	CPPcost = "float "+ actionName->getText() +"Cost(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"+ CPPConstparameters + "\nreturn "+ costFctName->getText() +"("; }
( argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2] { CPPcost += "(*"+argTypeAndCppCode.second+")"; }
( COMMA argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2] { CPPcost += ", (*"+argTypeAndCppCode.second+")"; }
)*
)?
RIGHT_PAREN { CPPcost += ");\n}\n\n"; }
RIGHT_SCOPE SEMICOLON
// duration ...
DURATION LEFT_SCOPE timeFctName:OTHER_ID LEFT_PAREN {
	CPPtime = "pair<double,double> "+ actionName->getText() +"Time(LightWorldBase const& wb, vector<LightEntity const*> const& args)\n{\n"+ CPPConstparameters +"\nreturn "+ timeFctName->getText() +"(";
}
( argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2] { CPPtime += "(*"+argTypeAndCppCode.second+")"; }
( COMMA argTypeAndCppCode=singleTerm[availableEntities,false,tmp1,tmp2] { CPPtime += ", (*"+argTypeAndCppCode.second+")"; }
)*
)?
RIGHT_PAREN { CPPtime += ");\n}\n\n"; }
RIGHT_SCOPE SEMICOLON
// write everything in memory ...
RIGHT_SCOPE {
	commActionRedefinedList[actionName->getText()] = CPPdef + CPPprecond + CPPeffect + CPPcost + CPPtime;
};

//---------------------------------------------------------------------------------------------------------------
// SOCIAL RULES DEFINITION
//---------------------------------------------------------------------------------------------------------------

//********* start socialRulesDef rule *********
socialRulesDef returns[std::pair<std::string, std::string> initAndMainCode] throws HATPParserException : {
	std::string initCode, mainCode;
	std::pair<std::string, std::string> InitAndCode;
	initCode = "#include<";
	initCode += SOCIALRULES_EXIT_FILE;
	initCode += ">\n\nusing namespace std;";
	initCode += "\n\nvoid initSocialRuleBase(SocialRuleBase& srb, PartialPlanAppraiser& ppa)\n{\nvector<string> vec;\n(void)vec;\n\n";
	mainCode = "\nusing namespace std;\nusing namespace HATP_TOOLS;\n\n";
}
( InitAndCode=socialRule {initCode += InitAndCode.first; mainCode += InitAndCode.second;} )*
{
	std::vector<std::vector<float> > matrix = std::vector<std::vector<float> >(priorities.size()+1, std::vector<float>(priorities.size()+1));
	std::map<std::string, int>::iterator it1, it2;
	// we complete the AHP matrix
	matrix[0][0] = 1.;
	for(it1 = priorities.begin() ; it1 != priorities.end() ; ++it1) {
		if(it1->second >= 0) {
			matrix[0][1+std::distance(priorities.begin(), it1)] = 1./(1. + (float)(it1->second));
			matrix[1+std::distance(priorities.begin(), it1)][0] = 1. + (float)(it1->second);
		} else {
			matrix[0][1+std::distance(priorities.begin(), it1)] = 1. - (float)(it1->second);
			matrix[1+std::distance(priorities.begin(), it1)][0] = 1./(1. - (float)(it1->second));
		}
	}
	float diff;
	for(it1 = priorities.begin() ; it1 != priorities.end() ; ++it1) {
		for(it2 = it1 ; it2 != priorities.end() ; ++it2) {
			if(it1 == it2) {
				matrix[1+std::distance(priorities.begin(), it1)][1+std::distance(priorities.begin(), it2)] = 1.;
			} else {
				diff = (float)(it1->second) - (float)(it2->second);
				if(diff >= 0.) {
					matrix[1+std::distance(priorities.begin(), it1)][1+std::distance(priorities.begin(), it2)] = 1. + diff;
					matrix[1+std::distance(priorities.begin(), it2)][1+std::distance(priorities.begin(), it1)] = 1./(1. + diff);
				}
				else {
					matrix[1+std::distance(priorities.begin(), it1)][1+std::distance(priorities.begin(), it2)] = 1./(1. - diff);
					matrix[1+std::distance(priorities.begin(), it2)][1+std::distance(priorities.begin(), it1)] = 1. - diff;
				}
			}
		}
	}
	// we means and we update matrix
	unsigned int i,j;
	std::vector<float> vecMeans(matrix.size(), 0.);
	for(i=0 ; i<matrix.size() ; ++i) {
		for(j=0 ; j<matrix.size() ; ++j) {
			vecMeans[j] += matrix[i][j];
		}
	}
	for(i=0 ; i<matrix.size() ; ++i) {
		for(j=0 ; j<matrix.size() ; ++j) {
			matrix[i][j] /= vecMeans[j];
		}
	}
	// we compute priorities in vecMeans
	vecMeans = std::vector<float>(matrix.size(), 0.);
	for(i=0 ; i<matrix.size() ; ++i) {
		for(j=0 ; j<matrix.size() ; ++j) {
			vecMeans[i] += matrix[i][j];
		}
	}
	for(i=0 ; i<vecMeans.size() ; ++i) {
		vecMeans[i] /= (float)matrix.size();
	}
	// we compute C++ code
	std::ostringstream st;
	st << std::endl << "map<string, float> scoreParameters;" << std::endl;
	st << "scoreParameters[COSTS_PART] = " << vecMeans[0] << ";" << std::endl;
	for(it1 = priorities.begin() ; it1 != priorities.end() ; ++it1) {
		if(it1->first == TIME_PART_TAG || it1->first == INTRICACY_CONTROL_TAG || it1->first == WASTED_TIME_TAG || it1->first == EFFORT_BALANCING_TAG) {
			st << "scoreParameters[" << it1->first << "] = " << vecMeans[1+std::distance(priorities.begin(), it1)] << ";" << std::endl;
		} else {
			st << "scoreParameters[\"" << it1->first << "\"] = " << vecMeans[1+std::distance(priorities.begin(), it1)] << ";" << std::endl;
		}
	}
	st << std::endl << "Plan::scoreWeights = scoreParameters;" << std::endl;
	initCode += st.str() + "\n}\n";
	initAndMainCode = std::make_pair(initCode, mainCode);
};
//********* end socialRulesDef rule *********

//********* start socialRule rule *********
socialRule returns[std::pair<std::string, std::string> InitAndCode] throws HATPParserException {
	static bool effortBalancingDefined=false;
	static bool intricacyControlDefined=false, wastedTimeDefined=false;
	InitAndCode = std::make_pair("","");
	std::string code;
	std::set<std::string> undesirableStateNames, undesirableSequenceNames, badDecompositionNames;
} :
// alternative 0: Time Part prioritie
TIME_PART LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON RIGHT_SCOPE {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[TIME_PART_TAG] = prNb;
}
| // alternative 1: undesirable state
UNDESIRABLE_STATE name1:OTHER_ID code=undesirableState[name1->getText()] {
	if(undesirableStateNames.find(name1->getText()) != undesirableStateNames.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Name \""+name1->getText()+"\" already used for an undesirable state");
	}
	undesirableStateNames.insert(name1->getText());
	InitAndCode.first = "srb.add(\""+name1->getText()+"\", "+name1->getText()+"Func);\n\n";
	InitAndCode.second += "float "+ name1->getText() +"Func(LightWorldBase const& wb, TimeProjection const& proj)\n{\n"+ code +"\n}\n\n";
}
| // alternative 2: undesirable sequence
{std::set<std::string> taskNames;}
UNDESIRABLE_SEQUENCE name2:OTHER_ID code=undesirableSequence[name2->getText(), taskNames] {
	if(undesirableSequenceNames.find(name2->getText()) != undesirableSequenceNames.end()) {
		throw HATPParserException(ERROR, name2->getLine(), "Name \""+name2->getText()+"\" already used for an undesirable sequence");
	}
	undesirableSequenceNames.insert(name2->getText());
	InitAndCode.first = "vec.clear();\n";
	for(std::set<std::string>::const_iterator itS = taskNames.begin() ; itS != taskNames.end() ; ++itS) {
		InitAndCode.first += "vec.push_back(\""+*itS+"\");\n";
	}
	InitAndCode.first += "ppa.addUndesirableSequence(\""+name2->getText()+"\", vec);\n";
	InitAndCode.first += "srb.add(\""+name2->getText()+"\", "+name2->getText()+"InitFunc, "+name2->getText()+"Func);\n\n";
	InitAndCode.second += code;
}
| // alternative 3: effort balancing
code=effortBalancing {
	 if(effortBalancingDefined) {
	 	throw HATPParserException(ERROR, 0, "Effort balancing rule defined several times.");
	 }
	 effortBalancingDefined = true;
	 InitAndCode.first = "srb.add(effortBalancing);\n\n";
	 InitAndCode.second += "float effortBalancing(LightWorldBase const& wb, map<string, float> const& efforts)\n{\n"+ code +"\n}\n\n";
}
| // alternative 4: control of intricacy
code=intricacyControl {
	if(intricacyControlDefined) {
		throw HATPParserException(ERROR, 0, "Control of Intricacy rule defined several times.");
	}
	intricacyControlDefined = true;
	InitAndCode.first = "srb.add(controlOfIntricacyInitFunc, controlOfIntricacyFunc);\n\n";
	InitAndCode.second += code;
}
| // alternative 6: bad decomposition
{std::set<std::string> taskNames;}
BAD_DECOMPO name3:OTHER_ID code=badDecomposition[name3->getText(), taskNames] {
	if(badDecompositionNames.find(name3->getText()) != badDecompositionNames.end()) {
		throw HATPParserException(ERROR, name3->getLine(), "Name \""+name2->getText()+"\" already used for an undesirable sequence");
	}
	badDecompositionNames.insert(name3->getText());
	InitAndCode.first = "vec.clear();\n";
	for(std::set<std::string>::const_iterator itS = taskNames.begin() ; itS != taskNames.end() ; ++itS) {
		InitAndCode.first += "vec.push_back(\""+*itS+"\");\n";
	}
	InitAndCode.first += "ppa.addBadDecomposition(\""+name3->getText()+"\", vec);\n";
	InitAndCode.first += "srb.add(\""+name3->getText()+"\", "+name3->getText()+"InitFunc, "+name3->getText()+"Func);\n\n";
	InitAndCode.second += code;
}
| // alternative 7: wasted time
code=wastedTime {
	if(wastedTimeDefined) {
		throw HATPParserException(ERROR, 0, "Wasted Time rule defined several times.");
	}
	wastedTimeDefined = true;
	InitAndCode.first = "srb.add(wasteOfTimeInitFunc, wasteOfTimeFunc);\n\n";
	InitAndCode.second += code;
};
//********* end socialRule rule *********

//******** start precondsReport rule ******** (for Undesirable State only) ?
precondsReport[std::map<std::string, std::pair<std::string, std::string> > availableEntities]
returns [std::pair<std::string, std::string> testsAndReport] throws HATPParserException {
	testsAndReport = std::make_pair("","");
	std::pair<std::vector<std::string>, std::string> termTypeAndCppCode1, termTypeAndCppCode2;
	std::string opCPPcode;
	unsigned int nbLine;
	std::string entityName1, attributeName1, entityName2, attributeName2;
} :
( {
	entityName1.clear();
	entityName2.clear();
	attributeName1.clear();
	attributeName2.clear();
}
termTypeAndCppCode1=singleTerm[availableEntities, false, entityName1, attributeName1]
( op1:TEST_EQUAL { opCPPcode="=="; nbLine=op1->getLine(); }
| op2:TEST_DIFF { opCPPcode="!="; nbLine=op2->getLine(); }
| op3:TEST_SET_IN { opCPPcode=">>"; nbLine=op3->getLine(); }
| op4:TEST_SET_NOT_IN { opCPPcode="^="; nbLine=op4->getLine(); }
| op5:SUP { opCPPcode=">"; nbLine=op5->getLine(); }
| op6:SUP_EQUAL { opCPPcode=">="; nbLine=op6->getLine(); }
| op7:INF { opCPPcode="<"; nbLine=op7->getLine(); }
| op8:INF_EQUAL { opCPPcode="<="; nbLine=op8->getLine(); }
)
termTypeAndCppCode2=singleTerm[availableEntities, false, entityName2, attributeName2]
SEMICOLON {
	// the two terms must be of the same kind
	if( !( termTypeAndCppCode1.first[2] == termTypeAndCppCode2.first[2] ||
	 (termTypeAndCppCode1.first[2] == NULL_ETYPE && termTypeAndCppCode2.first[2] != NUMBER_ETYPE
	&& termTypeAndCppCode2.first[2] != STRING_ETYPE && termTypeAndCppCode2.first[2] != BOOL_ETYPE ) ||
	 (termTypeAndCppCode2.first[2] == NULL_ETYPE && termTypeAndCppCode1.first[2] != NUMBER_ETYPE
	&& termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE )) ) {
		throw HATPParserException(ERROR, nbLine, "comparison must be done between terms of same kind");
	}
	if( (opCPPcode == "==" || opCPPcode == "!=") // term1 and term2 must be atoms
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE ) ) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between atoms");
	}
	if( (opCPPcode == ">>" || opCPPcode == "^=") // term1 must be an atom and term2 must be a set
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == SET_ETYPE ) ) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between an atom and a set");
	}
	if( (opCPPcode == ">" || opCPPcode == ">=" || opCPPcode == "<" || opCPPcode == "<=" )
	&& !( termTypeAndCppCode1.first[1] == ATOM_ETYPE && termTypeAndCppCode2.first[1] == ATOM_ETYPE
	&& termTypeAndCppCode1.first[2] == NUMBER_ETYPE && termTypeAndCppCode2.first[2] == NUMBER_ETYPE )) {
		throw HATPParserException(ERROR, nbLine, "this kind of test must be done between not bool atoms");
	}
	// everything is OK
	if(termTypeAndCppCode1.first[0] == STATIC_ETYPE && termTypeAndCppCode2.first[0] == STATIC_ETYPE) {
		// test between static elements, we add it in testsAndReport.first
		testsAndReport.first += termTypeAndCppCode1.second + opCPPcode + termTypeAndCppCode2.second + " && ";
	} else { // we complete report in testsAndReport.second
		std::string CPPval1 = termTypeAndCppCode1.second;
		std::string CPPval2 = termTypeAndCppCode2.second;
		bool testDiff;
		if(opCPPcode == "==" || opCPPcode == "!=" || opCPPcode == ">>" || opCPPcode == "^=") {
			if(attributeName1 != "" && termTypeAndCppCode1.first[0] == DYNAMIC_ETYPE) {
			// first term is obtained from an entity and an attribute
				testDiff = false;
				testsAndReport.second += "report.";
				if( opCPPcode == "==" ) {
					testsAndReport.second += "addAtomTestEqual";
				} else if( opCPPcode == ">>" ) {
					testsAndReport.second += "addVectorTestIn";
				} else if( opCPPcode == "^=" ) {
					testsAndReport.second += "addVectorTestNotIn";
				} else if( opCPPcode == "!=" ) {
					testsAndReport.second += "addAtomTestDiff";
					testDiff = true;
				}
				if(!testDiff) {
					if(termTypeAndCppCode2.first[2] != NUMBER_ETYPE && termTypeAndCppCode2.first[2] != STRING_ETYPE && termTypeAndCppCode2.first[2] != BOOL_ETYPE){
						testsAndReport.second += "("+entityName1+"->getName(), \""+attributeName1+"\", ("+CPPval2+")?("+CPPval2+"->getName()):string(\"NULL\"))";
					} else {
						testsAndReport.second += "("+entityName1+"->getName(), \""+attributeName1+"\", "+CPPval2+")";
					}
				} else { // we store in the report the current value of entity1.attribute1 to maintain precond at true
					if(termTypeAndCppCode1.first[2] != NUMBER_ETYPE && termTypeAndCppCode1.first[2] != STRING_ETYPE && termTypeAndCppCode1.first[2] != BOOL_ETYPE){
						testsAndReport.second += "("+entityName1+"->getName(), \""+attributeName1+"\", ("+CPPval1 +")?("+CPPval1+"->getName()):string(\"NULL\"))";
					} else {
						testsAndReport.second += "("+entityName1+"->getName(), \""+attributeName1+"\", "+CPPval1+")";
					}
				}
				testsAndReport.second += ";\n";
			}
			if(attributeName2 != "" && termTypeAndCppCode2.first[0] == DYNAMIC_ETYPE) {
			// second term is obtained from an entity and an attribute
				testDiff = false;
				testsAndReport.second += "report.";
				if( opCPPcode == "==" ) {
					testsAndReport.second += "addAtomTestEqual";
				} else if( opCPPcode == ">>" ) {
					testsAndReport.second += "addVectorTestIn";
				} else if( opCPPcode == "^=" ) {
					testsAndReport.second += "addVectorTestNotIn";
				} else if( opCPPcode == "!=" ) {
					testsAndReport.second += "addAtomTestDiff";
					testDiff = true;
				}
				if(!testDiff) {
					if(termTypeAndCppCode1.first[2] != NUMBER_ETYPE && termTypeAndCppCode1.first[2] != STRING_ETYPE &&
				termTypeAndCppCode1.first[2] != BOOL_ETYPE) {
						testsAndReport.second += "("+entityName2+"->getName(), \""+attributeName2+"\", ("+CPPval1
				+")?("+CPPval1+"->getName()):string(\"NULL\"))";
					} else {
						testsAndReport.second += "("+entityName2+"->getName(), \""+attributeName2+"\", "+CPPval1+")";
					}
				} else {// we store in the report the current value of entity2.attribute2 to maintain precond at true
					if(termTypeAndCppCode2.first[2] != NUMBER_ETYPE && termTypeAndCppCode2.first[2] != STRING_ETYPE &&
				termTypeAndCppCode2.first[2] != BOOL_ETYPE) {
						testsAndReport.second += "("+entityName2+"->getName(), \""+attributeName2+"\", ("+CPPval2
				+")?("+CPPval2+"->getName()):string(\"NULL\"))";
					} else {
						testsAndReport.second += "("+entityName2+"->getName(), \""+attributeName2+"\", "+CPPval2+")";
					}
				}
				testsAndReport.second += ";\n";
			}
		}
	} // end else : we complete report
}
)*
{ testsAndReport.first += "true"; } ;
//********* end precondsReport rule *********


//********* end undesirableState rule *********
// return the contents C++ code of the function describing the application of the social rule
undesirableState[std::string name] returns[std::string code] throws HATPParserException : {
	current_task_parameters.clear();
	current_task_select_terms.clear();
	std::string tmp, CPPloopsCode, iterator, sType;
	std::ostringstream st;
	std::pair<std::string, std::string> testsAndReport;
	unsigned int i;
	code = "WorldBaseReport report;\n(void) report;\n";
	code += "bool posState, atEnd;\nSubstitution* subst;\nvector<string> entitiesVector;\n";
	code += "vector<double> durations;\n";
	code += "vector<UndesirableStateData*> vecData;\n\n";
}
LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try{
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[name] = prNb;
}
( type:ENTITY_ID {
		sType = type->getText();
		if(definedEntityTypes.find(sType) == definedEntityTypes.end()) {
			throw HATPParserException(ERROR, type->getLine(), "Entity type " + type->getText() + " unknown");
		}
	}
	name1:ENTITY_ID {
		for(i=0 ; i<current_task_select_terms.size() ; ++i) {
			if(current_task_select_terms[i].first == name1->getText()) {
				throw HATPParserException(ERROR, name1->getLine(), "Entity " + name1->getText() + " already defined");
			}
		}
		current_task_select_terms.push_back(std::make_pair(name1->getText(),sType));
		iterator = name1->getText()+"_iterator";
		current_task_select_terms_code.push_back(name1->getText()+"["+iterator+"]");
		code += "entitiesVector.push_back(\""+name1->getText()+"\");\n";
		CPPloopsCode += "for(unsigned int "+iterator+"=0 ; "+iterator+"<"+ name1->getText() +".size() ; ++"+ iterator +")\n{\n";
	}
	(COMMA name2:ENTITY_ID {
		 for(i=0 ; i<current_task_select_terms.size() ; ++i) {
			 if(current_task_select_terms[i].first == name2->getText()) {
				throw HATPParserException(ERROR, name2->getLine(), "Entity " + name2->getText() + " already defined");
			}
		}

		current_task_select_terms.push_back(std::make_pair(name2->getText(),sType));
		iterator = name2->getText()+"_iterator";
		current_task_select_terms_code.push_back(name2->getText()+"["+iterator+"]");
		code += "entitiesVector.push_back(\""+name2->getText()+"\");\n";
		CPPloopsCode += "for(unsigned int "+iterator+"=0 ; "+iterator+"<"+ name2->getText() +".size() ; ++"+ iterator +")\n{\n";
	 } )* SEMICOLON
)+ {
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		st.str(""); // we clear st
		st << "vector<LightEntity*> " << current_task_select_terms[i].first; // variable name <select term name>_0
		st << " = wb.findByType(\"" << current_task_select_terms[i].second << "\");\n";
		code += st.str();
	}
	code += CPPloopsCode;
	code += "report.clearAll();\n";
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	this->completeMapWithTaskSelectTerms(availableEntities);
}
//CONDITIONS LEFT_SCOPE codeWithAndWithoutReports=preconds[availableEntities, false] RIGHT_SCOPE
CONDITIONS LEFT_SCOPE testsAndReport=precondsReport[availableEntities] RIGHT_SCOPE {
	code += "if("+testsAndReport.first+")\n{\n";
	code += testsAndReport.second;
	code += "subst = new Substitution(entitiesVector);\n";
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		code += "subst->setValue(\""+current_task_select_terms[i].first+"\", "+current_task_select_terms_code[i]+"->getName());\n";
	}
	code += "durations.clear();\n";
	code += "\nposState=proj.possibleState(report, wb, durations, atEnd);\nif(posState)\n";
	code += "vecData.push_back(new UndesirableStateData(subst, durations, atEnd));\n";
	code += "else\ndelete subst;\n";
}
PENALTY LEFT_SCOPE penFctName:OTHER_ID RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {
	code += "}\n";
	//we close the for loops associated to selectTerms
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		code += "}\n";
	}
	code += "float totalCost=0.;\n";
	code += "if(!vecData.empty())\n";
	code += "totalCost = "+ penFctName->getText() +"(vecData, proj.totalDuration(), wb);\n\n";
	code += "for(unsigned int j=0 ; j<vecData.size() ; ++j)\ndelete vecData[j];\n\n";
	code += "return totalCost;";
	// undesirable state description is finished: we reset select terms
	current_task_select_terms.clear();
	current_task_select_terms_code.clear();
} ;
//********* end undesirableState rule *********

//********* start undesirableSequence rule *********
// return the contents C++ code of the function describing the application of the social rule
undesirableSequence[std::string name, std::set<std::string>& usedTasks]
returns[std::string code] throws HATPParserException : {
	current_task_parameters.clear();
	current_task_select_terms.clear();
	current_task_select_terms_code.clear();
	unsigned int i;
	std::string sType;
	code = "pair<TaskList*, vector<string> > "+name+"InitFunc()\n{\n";
	code += "map<string,string> entitiesMap;\nvector<string> entitiesVector;\n\n";
	std::map<std::string,std::string> availableArguments;
	std::set<std::string> usedArguments;
}
LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[name] = prNb;
}
( type:ENTITY_ID {
		sType = type->getText();
		if(definedEntityTypes.find(sType) == definedEntityTypes.end()) {
			throw HATPParserException(ERROR, type->getLine(), "Entity type " + type->getText() + " unknown");
		}
	}
	name1:ENTITY_ID {
		for(i=0 ; i<current_task_select_terms.size() ; ++i) {
			if(current_task_select_terms[i].first == name1->getText()) {
				throw HATPParserException(ERROR, name1->getLine(), "Entity " + name1->getText() + " already defined");
			}
		}
		current_task_select_terms.push_back(std::make_pair(name1->getText(),sType));
		current_task_select_terms_code.push_back(name1->getText());
		availableArguments[name1->getText()] = sType;
		code += "entitiesMap[\""+name1->getText()+"\"] = \""+sType+"\";\n";
		code += "entitiesVector.push_back(\""+name1->getText()+"\");\n";
	}
	(COMMA name2:ENTITY_ID {
		for(i=0 ; i<current_task_select_terms.size() ; ++i) {
			if(current_task_select_terms[i].first == name2->getText()) {
				throw HATPParserException(ERROR, name2->getLine(), "Entity " + name2->getText() + " already defined");
			}
		}
		current_task_select_terms.push_back(std::make_pair(name2->getText(),sType));
		current_task_select_terms_code.push_back(name2->getText());
		availableArguments[name2->getText()] = sType;
		code += "entitiesMap[\""+name2->getText()+"\"] = \""+sType+"\";\n";
		code += "entitiesVector.push_back(\""+name2->getText()+"\");\n";
	 } )* SEMICOLON
)+ {
	code += "\n";
	code += "vector<string> tmp_args;\n(void)tmp_args;\n";
	code += "map<unsigned int, TaskInstance*> map_tasks;\n(void)map_tasks;\n";
	code += "map<unsigned int, vector<unsigned int> > map_links;\n(void)map_links;\n";
	code += "map<unsigned int, vector<string> > map_linkTypes;\n(void)map_linkTypes;\n\n";
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	this->completeMapWithTaskSelectTerms(availableEntities);
	std::pair<std::string, std::string> codeWithAndWithoutReports;
	std::string tmp;
}
CONDITIONS LEFT_SCOPE codeWithAndWithoutReports=preconds[availableEntities,false] RIGHT_SCOPE
SEQUENCE LEFT_SCOPE tmp=sequenceDescription[false, 0, availableArguments, usedArguments, usedTasks, true] rs:RIGHT_SCOPE {
	// we check that all members of availableArguments have been used
	if(availableArguments.size() != usedArguments.size()) {
		throw HATPParserException(ERROR, rs->getLine(), "All defined entities must be used in the description");
	}
	code += tmp;
	code += "return make_pair(new TaskList(map_tasks, map_links, map_linkTypes), entitiesVector);\n}\n\n";
	code += "float "+name+"Func(TaskList const& subTL, vector<string> const& entitiesVector,";
	code += " LightWorldBase const& wb, TaskList const& proj, TimeProjection const& tproj)\n{\n";
	code += "unsigned int i,j;\n";
	code += "vector<Substitution*> goodSub, vecSub = proj.getSubstitutionsOnNodes(subTL, entitiesVector);\n";
	code += "for(i=0 ; i<vecSub.size() ; ++i)\n{\n";
	code += "map<string,pair<bool,string> > const& substitutionMap = vecSub[i]->getVariables();\n";
	for(unsigned int i=0 ; i<current_task_select_terms.size() ; ++i) {
		code += "LightEntity const* "+current_task_select_terms[i].first;
		code += " = wb.find((substitutionMap.find(\""+current_task_select_terms[i].first;
		code += +"\")->second).second,\""+current_task_select_terms[i].second+"\");\n(void)"+current_task_select_terms[i].first+";\n";
	}
	code += "if("+codeWithAndWithoutReports.second+")\n{\n";
	code += "for(j=0 ; j<goodSub.size() ; ++j)\n";
	code += "if(*(goodSub[j]) == *(vecSub[i]))\nbreak;\n\n";
	code += "if(j >= goodSub.size())\n";
	code += "goodSub.push_back(vecSub[i]);\n";
	code += "else\ndelete vecSub[i];\n";
	code += "}\nelse\n";
	code += "delete vecSub[i];\n}\n\n";
	code += "vector<SocialRuleData*> vecData = tproj.checkSubList(goodSub, wb, subTL);\n";
}
PENALTY LEFT_SCOPE penFctName:OTHER_ID {
	code += "float totalCost = 0.;\n";
	code += "if(!vecData.empty())\n";
	code += "totalCost = "+ penFctName->getText() +"(vecData, wb);\n\n";
	code += "for(i=0 ; i<vecData.size() ; ++i)\ndelete vecData[i];\n\n";
}
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {
	code += "return totalCost;\n}\n\n";
	// undesirable sequence description is finished: we reset select terms
	current_task_parameters.clear();
	current_task_select_terms.clear();
	current_task_select_terms_code.clear();
};
//********* end undesirableSequence rule *********

//******** start sequenceDescription rule ********
//syntax: <nb> : <task_name>(<arg1>, ..., <argn>) > <nb1>, > <nb2>, ... ;
sequenceDescription[bool printInBadDecompositionNodes, unsigned int currentNode,
	std::map<std::string,std::string> const& availableArguments, std::set<std::string>& usedArguments,
	std::set<std::string>& usedTasks, bool mustBeActions] returns[std::string code] throws HATPParserException :
{
	std::ostringstream st;
	std::string nb_string, op;
	std::pair<std::vector<std::string>, std::string> argsTypesAndCPPcode;
	current_decomposition_nodes.clear();
	unsigned int nb, tmp_nb;
	std::multimap<unsigned int,unsigned int>::iterator itBadDecompoNodes;
	std::map<std::string, std::string>::const_iterator itAvArgs;
	std::vector<std::string> argTypes, taskArgTypes;
	unsigned int i;
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	this->completeMapWithTaskSelectTerms(availableEntities);
}
(id:NUMBER_ID {
	// id must be an unsigned int, we use boost::lexical_cast
	try {
		nb = boost::lexical_cast<unsigned int>(id->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, id->getLine(), "ID of decomposition node " + id->getText() + " must be an unsigned int");
	}
	// id must have not been not declared before
	if(current_decomposition_nodes.find(nb) != current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, id->getLine(), "ID " + id->getText() + " has already been used in this decomposition");
	}
	if(printInBadDecompositionNodes) {
		for(itBadDecompoNodes = badDecompositionNodes.begin() ; itBadDecompoNodes != badDecompositionNodes.end() ; ++itBadDecompoNodes) {
			if(itBadDecompoNodes->first == currentNode && itBadDecompoNodes->second == nb) {
				throw HATPParserException(ERROR, id->getLine(), "ID " + id->getText() + " has already been used in this decomposition");
			}
		}
		badDecompositionNodes.insert(std::make_pair(currentNode, nb));
	}
	// we do not add nb in current_decomposition_nodes now because we have to check links validity before
	// we get nb under a string form to produce C++ code
	st.str("");
	st << nb;
	nb_string = st.str();
}
COLON taskName:ENTITY_ID {
	usedTasks.insert(taskName->getText());
	code += "map_links["+ nb_string +"] = vector<unsigned int>(0);\ntmp_args.clear();\n";
	code += "map_linkTypes["+ nb_string +"] = vector<string>(0);\n";
	// we check if taskName exists
	std::map<std::string, bool>::iterator it=htnTasksIsAction.find(taskName->getText());
	if(it == htnTasksIsAction.end()) {
		throw HATPParserException(ERROR, taskName->getLine(), "Task " + taskName->getText() + " unknown");
	}
	if(mustBeActions && !(it->second)) {
		throw HATPParserException(ERROR, taskName->getLine(), "Task " + taskName->getText() + " must be an action");
	}
	argTypes.clear();
}
LEFT_PAREN
( arg:ENTITY_ID {
		itAvArgs = availableArguments.find(arg->getText());
		if(itAvArgs == availableArguments.end()) {
			throw HATPParserException(ERROR, arg->getLine(), "unknown parameter "+arg->getText());
		}
		code += "tmp_args.push_back(\""+arg->getText()+"\");\n";
		usedArguments.insert(arg->getText());
		argTypes.push_back(itAvArgs->second);
	}
	( COMMA arg1:ENTITY_ID {
			itAvArgs = availableArguments.find(arg1->getText());
			if(itAvArgs == availableArguments.end()) {
				throw HATPParserException(ERROR, arg1->getLine(), "unknown parameter "+arg1->getText());
			}
			code += "tmp_args.push_back(\""+arg1->getText()+"\");\n";
			usedArguments.insert(arg1->getText());
			argTypes.push_back(itAvArgs->second);
		}
	)*
)? RIGHT_PAREN {
	// we check arguments compatibility
	taskArgTypes = htnTasks[taskName->getText()];
	if(argTypes.size() != taskArgTypes.size()) {
		throw HATPParserException(ERROR, arg1->getLine(), "bad number of arguments for task "+taskName->getText());
	} else {
		for(i=0 ; i<argTypes.size() ; ++i) {
			if(argTypes[i] != taskArgTypes[i]) {
				st.str("");
				st << i;
				throw HATPParserException(ERROR, arg1->getLine(), "argument "+st.str()+" for task "+taskName->getText()+" has a bad type");
				break;
			}
		}
	}
	code += "map_tasks["+ nb_string +"] = new TaskInstance(\""+ taskName->getText() +"\", tmp_args, entitiesMap);\n";
}
( ( (SUP { op = "IMMEDIATE_LINK"; } | SUP_TILD { op = "LATER_LINK"; } ) idNext:NUMBER_ID COMMA {
	// idNext must be an unsigned int (we use boost::lexical_cast) and must have been declared before
	try {
		tmp_nb = boost::lexical_cast<unsigned int>(idNext->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, idNext->getLine(), "ID of decomposition node " + idNext->getText() + " must be an unsigned int");
	}
	// id must have been declared before
	if(current_decomposition_nodes.find(tmp_nb) == current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, idNext->getLine(), "ID " + idNext->getText() + " unknown at this point");
	}
	code += "map_links["+ nb_string +"].push_back("+ idNext->getText() +");\n";
	code += "map_linkTypes["+ nb_string +"].push_back("+ op +");\n";
}
)* (SUP { op = "IMMEDIATE_LINK"; } | SUP_TILD { op = "LATER_LINK"; } ) idNextEnd:NUMBER_ID {
	// idNextEnd must be an unsigned int (we use boost::lexical_cast) and must have been declared before
	try {
		tmp_nb = boost::lexical_cast<unsigned int>(idNextEnd->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, idNextEnd->getLine(), "ID of decomposition node " + idNextEnd->getText() + " must be an unsigned int");
	}
	// id must have been declared before
	if(current_decomposition_nodes.find(tmp_nb) == current_decomposition_nodes.end()) {// the node number has already been used in this decomposition
		throw HATPParserException(ERROR, idNextEnd->getLine(), "ID " + idNextEnd->getText() + " unknown at this point");
	}
	code += "map_links["+ nb_string +"].push_back("+ idNextEnd->getText() +");\n";
	code += "map_linkTypes["+ nb_string +"].push_back("+ op +");\n";
}
)?
SEMICOLON {
	// everything is OK in this line of decomposition description, we add nb in current_decomposition_nodes
	current_decomposition_nodes.insert(nb);
	code += "\n";
}
)+ ;
//********* end sequenceDescription rule *********

//********* end effortBalancing rule *********
// return the contents C++ code of the function describing the application of the social rule
effortBalancing returns[std::string code] throws HATPParserException : {
	code = "vector<LightEntity const*> agents;\n\n";
}
EFFORT_BALANCING LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
priorities[EFFORT_BALANCING_TAG] = prNb;
}
AGENTS EQUAL LEFT_SCOPE (name1:ENTITY_ID {
	// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name1->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name1->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name1->getText() + "\" must be of type Agent");
	}
	code += "agents.push_back(wb.find(\""+ name1->getText() +"\"));\n";
}
( COMMA name2:ENTITY_ID {
				// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name2->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name2->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name2->getText() + "\" must be of type Agent");
	}
	code += "agents.push_back(wb.find(\""+ name2->getText() +"\"));\n";
})*)
RIGHT_SCOPE SEMICOLON {
	code += "map<LightEntity const*, float> agsEff;\n\n";
	code += "for(unsigned int iterator=0 ; iterator<agents.size() ; ++iterator){\n";
	code += "if( efforts.find(agents[iterator]->getName()) != efforts.end() )\n";
	code += "agsEff[agents[iterator]] = efforts.find(agents[iterator]->getName())->second;\n";
	code += "else\n";
	code += "agsEff[agents[iterator]] = 0;\n";
	code += "}\n";
}
PENALTY LEFT_SCOPE penFctName:OTHER_ID {
code += "return "+ penFctName->getText() +"(agsEff);\n"; }
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {

};
//********* end effortBalancing rule *********

//********* end intricacyControl rule *********
// return the contents C++ code of the function describing the application of the social rule
intricacyControl returns[std::string code] throws HATPParserException : {
	code = "set<string> controlOfIntricacyInitFunc()\n{\nset<string> agents;\n\n";
}
INTRICACY_CONTROL LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[INTRICACY_CONTROL_TAG] = prNb;
}
AGENTS EQUAL LEFT_SCOPE (name1:ENTITY_ID {
	// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name1->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name1->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name1->getText() + "\" must be of type Agent");
	}
	code += "agents.insert(\""+ name1->getText() +"\");\n";
}
( COMMA name2:ENTITY_ID {
	// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name2->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name2->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name2->getText() + "\" must be of type Agent");
	}
	code += "agents.insert(\""+ name2->getText() +"\");\n";
})*)
RIGHT_SCOPE SEMICOLON {
	code += "\nreturn agents;\n}\n\nfloat controlOfIntricacyFunc(TaskList const& projection, set<string> const& agents)\n{\n"; }
PENALTY LEFT_SCOPE penFctName:OTHER_ID {
	code += "return "+ penFctName->getText() +"( projection.getNbIntricateLinks(agents) );\n}\n\n"; }
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {
} ;
//********* end intricacyControl rule *********

//********* start badDecomposition rule *********
// return the contents C++ code of the function describing the application of the social rule
badDecomposition[std::string name, std::set<std::string>& usedTasks]
returns[std::string code] throws HATPParserException : {
	current_task_parameters.clear();
	current_task_select_terms.clear();
	current_task_select_terms_code.clear();
	std::ostringstream st;
	std::string tmp, sType;
	std::pair<std::string, std::string> eTypeAndCPPcode;
	unsigned int i;
	std::map<std::string, std::string> availableArguments;
	std::set<std::string> usedArguments;
	code = "badDecompoDescription "+name+"InitFunc()\n{\n";
	code += "map<string,string> entitiesMap;\nvector<string> entitiesVector;\n\n";
}
LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[name] = prNb;
}
( type:ENTITY_ID {
	sType = type->getText();
	if(definedEntityTypes.find(sType) == definedEntityTypes.end()) {
		throw HATPParserException(ERROR, type->getLine(), "Entity type " + type->getText() + " unknown");
	}
}
	name1:ENTITY_ID {
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		if(current_task_select_terms[i].first == name1->getText()) {
			throw HATPParserException(ERROR, name1->getLine(), "Entity " + name1->getText() + " already defined");
		}
	}
	current_task_select_terms.push_back(std::make_pair(name1->getText(),sType));
	current_task_select_terms_code.push_back(name1->getText());
	availableArguments[name1->getText()] = sType;
	code += "entitiesMap[\""+name1->getText()+"\"] = \""+sType+"\";\n";
	code += "entitiesVector.push_back(\""+name1->getText()+"\");\n";
	}
	(COMMA name2:ENTITY_ID {
	for(i=0 ; i<current_task_select_terms.size() ; ++i) {
		if(current_task_select_terms[i].first == name2->getText()) {
			throw HATPParserException(ERROR, name2->getLine(), "Entity " + name2->getText() + " already defined");
		}
	}
	current_task_select_terms.push_back(std::make_pair(name2->getText(),sType));
	current_task_select_terms_code.push_back(name2->getText());
	availableArguments[name2->getText()] = sType;
	code += "entitiesMap[\""+name2->getText()+"\"] = \""+sType+"\";\n";
	code += "entitiesVector.push_back(\""+name2->getText()+"\");\n";
	}
)* SEMICOLON
)+ {
	code += "\n";
	code += "vector<string> tmp_args;\n(void)tmp_args;\n";
	code += "map<unsigned int, TaskInstance*> map_tasks;\n(void)map_tasks;\n";
	code += "map<unsigned int, vector<unsigned int> > map_links;\n(void)map_links;\n";
	code += "map<unsigned int, vector<string> > map_linkTypes;\n(void)map_linkTypes;\n";
	code += "TaskList* subTL;\n(void)subTL;\n\n";
	code += "vector<TaskList*> subPTnodes;\n(void)subPTnodes;\n";
	code += "map<TaskListNode*, pair<TaskList*, string> > hierarchical_links;\n(void)hierarchical_links;\n";
	code += "map<unsigned int, TaskList*> BadDecompoNodes;\n(void)BadDecompoNodes;\n";
	code += "map<unsigned long int, unsigned int> tmpIDcorrespondence;\n(void)tmpIDcorrespondence;\n";
	code += "map<pair<unsigned int, unsigned int>, TaskListNode*> IDcorrespondence;\n(void)IDcorrespondence;\n";
	code += "map<unsigned long int, unsigned int>::const_iterator it_corres;\n\n";
	std::map<std::string, std::pair<std::string, std::string> > availableEntities;
	this->completeMapWithTaskSelectTerms(availableEntities);
	std::pair<std::string, std::string> codeWithAndWithoutReports;
}
CONDITIONS LEFT_SCOPE codeWithAndWithoutReports=preconds[availableEntities,false] RIGHT_SCOPE
DECOMPOSITION LEFT_SCOPE tmp=badDecompositionDescription[availableArguments, usedArguments, usedTasks] rs:RIGHT_SCOPE {
	// we check that all members of availableArguments have been used
	if(availableArguments.size() != usedArguments.size()) {
		throw HATPParserException(ERROR, rs->getLine(), "All defined entities must be used in the description");
	}
	code += tmp+"\n";
	code += "return make_pair(subPTnodes, make_pair(hierarchical_links, entitiesVector));\n";
	code += "}\n\n";
	code += "float "+name+"Func(vector<TaskList*> const& subPTnodes, ";
	code += "map<TaskListNode*, pair<TaskList*, string> > const& hierarchical_links, ";
	code += "vector<string> const& entitiesVector, LightWorldBase const& wb, PlanTree const& tree)\n{\n";
	code += "unsigned int i,j;\n";
	code += "vector<Substitution*> goodSub, vecSub=tree.getSubstitutionsOnNodes(subPTnodes, entitiesVector);\n";
	code += "for(i=0 ; i<vecSub.size() ; ++i)\n{\n";
	code += "map<string,pair<bool,string> > const& substitutionMap = vecSub[i]->getVariables();\n";
	for(unsigned int i=0 ; i<current_task_select_terms.size() ; ++i) {
		code += "LightEntity const* "+current_task_select_terms[i].first;
		code += " = wb.find((substitutionMap.find(\""+current_task_select_terms[i].first;
		code += +"\")->second).second,\""+current_task_select_terms[i].second+"\");\n(void)"+current_task_select_terms[i].first+";\n";
	}
	code += "\nif("+codeWithAndWithoutReports.second+")\n{\n";
	code += "for(j=0 ; j<goodSub.size() ; ++j)\n";
	code += "if(*(goodSub[j]) == *(vecSub[i]))\nbreak;\n\n";
	code += "if(j >= goodSub.size())\n";
	code += "goodSub.push_back(vecSub[i]);\n";
	code += "else\ndelete vecSub[i];\n";
	code += "}\nelse\n";
	code += "delete vecSub[i];\n}\n\n";
	code += "vector<SocialRuleData*> vecData = tree.checkSubTree2(wb, goodSub, subPTnodes, hierarchical_links, entitiesVector);\n";
}
PENALTY LEFT_SCOPE penFctName:OTHER_ID {
	code += "float totalCost=0.;\n";
	code += "if(!vecData.empty())\n";
	code += "totalCost = "+penFctName->getText() +"(vecData, wb);\n\n";
}
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {
	code += "for(i=0 ; i<vecData.size() ; ++i)\ndelete vecData[i];\n\n";
	code += "return totalCost;\n}\n\n";
	current_task_parameters.clear();
	current_task_select_terms.clear();
	current_task_select_terms_code.clear();
} ;
//********* end badDecomposition rule *********

//******** start badDecompositionDescription rule ********
//return C++ code associated to the bad decomposition
badDecompositionDescription[std::map<std::string,std::string> availableArguments, std::set<std::string>& usedArguments,
	std::set<std::string>& usedTasks] returns[std::string code] throws HATPParserException : {
	std::string tmp, linkType;
	unsigned int nb, nb2, nb3, nb_son;
	std::ostringstream st;
	badDecompositionNodes.clear(); //std::multimap<unsigned int,unsigned int> badDecompositionNodes;
	std::multimap<unsigned int, unsigned int>::iterator itBadDecompo;
	bool nodeExists;
}
( {
	 code += "map_links.clear();\n";
	 code += "map_linkTypes.clear();\n";
	 code += "map_tasks.clear();\n";
	 code += "tmpIDcorrespondence.clear();\n";
}
NODE COLON nodeID:NUMBER_ID LEFT_SCOPE {
	try {
		nb = boost::lexical_cast<unsigned int>(nodeID->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, nodeID->getLine(), "ID of bad decomposition node " + nodeID->getText() + " must be an unsigned int");
	}
	if(badDecompositionNodes.find(nb) != badDecompositionNodes.end()) {// id node has been previously used
		throw HATPParserException(ERROR, nodeID->getLine(), "ID " + nodeID->getText() + " has already been used in bad decomposition description");
	}
}
tmp=sequenceDescription[true, nb, availableArguments, usedArguments, usedTasks, false] RIGHT_SCOPE {
	st.str("");
	st << nb;
	code += tmp;
	code += "subTL = new TaskList(map_tasks, map_links, map_linkTypes, &tmpIDcorrespondence);\n";
	code += "for(it_corres = tmpIDcorrespondence.begin() ; it_corres != tmpIDcorrespondence.end() ; ++it_corres)\n";
	code += "IDcorrespondence[make_pair("+ st.str() +",it_corres->second)] = subTL->find(it_corres->first);\n";
	code += "BadDecompoNodes["+ st.str() +"] = subTL;\n";
	code += "subPTnodes.push_back(subTL);\n\n\n";
}
)+
( nodeID2:NUMBER_ID COLON nodeID3:NUMBER_ID COLON {
		try {
			nb2 = boost::lexical_cast<unsigned int>(nodeID2->getText());
		} catch(boost::bad_lexical_cast &) {
			throw HATPParserException(ERROR, nodeID2->getLine(), "ID of bad decomposition node " + nodeID2->getText() + " must be an unsigned int");
		}
		try {
			nb3 = boost::lexical_cast<unsigned int>(nodeID3->getText());
		} catch(boost::bad_lexical_cast &) {
			throw HATPParserException(ERROR, nodeID3->getLine(), "ID of bad decomposition node " + nodeID3->getText() + " must be an unsigned int");
		}
		// the couple nodeID2 - TLnodeID must be in badDecompositionNodes
		nodeExists = false;
		for(itBadDecompo = badDecompositionNodes.begin() ; itBadDecompo != badDecompositionNodes.end() ; ++itBadDecompo){
			if(itBadDecompo->first == nb2 && itBadDecompo->second == nb3) {
				nodeExists = true;
				break;
			}
		}
		if( !nodeExists ) {
			throw HATPParserException(ERROR, nodeID2->getLine(), "Node with ID "+ nodeID2->getText() +":"+ nodeID3->getText() +" is unknown");
		}
	}
	( IMMEDIATE_DECOMPO { linkType = "IMMEDIATE_DECOMPOSITION"; } | DEEP_DECOMPO { linkType = "DEEP_DECOMPOSITION"; } )
	EQUAL NODE SonNodeID:NUMBER_ID SEMICOLON {
		try {
			nb_son = boost::lexical_cast<unsigned int>(SonNodeID->getText());
		} catch(boost::bad_lexical_cast &) {
			throw HATPParserException(ERROR, SonNodeID->getLine(), "ID of bad decomposition node " + SonNodeID->getText() + " must be an unsigned int");
		}
		// SonNodeID must be in badDecompositionNodes
		if(badDecompositionNodes.find(nb_son) == badDecompositionNodes.end()) {// id node is unknown
			throw HATPParserException(ERROR, SonNodeID->getLine(), "ID " + SonNodeID->getText() + " unknown in the current bad decomposition description");
		}
		code += "hierarchical_links[IDcorrespondence[make_pair(";
		st.str("");
		st << nb2;
		code += st.str() +",";
		st.str("");
		st << nb3;
		code += st.str() +")]] = make_pair(BadDecompoNodes[";
		st.str("");
		st << nb_son;
		code += st.str() +"],"+ linkType +");\n";
	}
)+ {
	badDecompositionNodes.clear();
} ;
//********* end badDecompositionDescription rule *********

//********* end wastedTime rule *********
// return the contents C++ code of the function describing the application of the social rule
wastedTime returns[std::string code] throws HATPParserException : {
	 code = "set<string> wasteOfTimeInitFunc()\n{\nset<string> agents;\n\n";
}
WASTED_TIME LEFT_SCOPE PRIORITY EQUAL pr:NUMBER_ID SEMICOLON {
	// pr must be an unsigned int between -8 and 8 included
	int prNb;
	try {
		prNb = boost::lexical_cast<int>(pr->getText());
	} catch(boost::bad_lexical_cast &) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer");
	}
	if(prNb%9 != prNb) {
		throw HATPParserException(ERROR, pr->getLine(), "Priority factor \"" + pr->getText() + "\" must be an integer between -8 and 8");
	}
	priorities[WASTED_TIME_TAG] = prNb;
}
AGENTS EQUAL LEFT_SCOPE (name1:ENTITY_ID {
	// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name1->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name1->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name1->getText() + "\" must be of type Agent");
	}
	code += "agents.insert(\""+ name1->getText() +"\");\n";
 }
( COMMA name2:ENTITY_ID {
				// name1 must be an existing entity of type Agent
	std::map<std::string, std::string>::const_iterator it = entities.find(name2->getText());
	if(it == entities.end()) {
		throw HATPParserException(ERROR, name1->getLine(), "Unknown entity \"" + name2->getText() + "\"");
	}
	if(it->second != AGENT) {
		throw HATPParserException(ERROR, name1->getLine(), "Entity \"" + name2->getText() + "\" must be of type Agent");
	}
	code += "agents.insert(\""+ name2->getText() +"\");\n";
})*)
RIGHT_SCOPE SEMICOLON {
	code += "\nreturn agents;\n}\n\n";
	code += "float wasteOfTimeFunc(TaskList const& projection, TimeProjection const& canvas, set<string> const& agents)\n{\n";
	code += "vector<double> durations;\nvector<WastedTimeData*> vecData;\ndouble fromStart, toEnd;\n\n";
	code += "for(set<string>::const_iterator it =agents.begin() ; it != agents.end() ; ++it)\n{\n";
	code += "durations = canvas.inactiveDuration(projection.getOrderedTaskIDs(*it), fromStart, toEnd);\n";
	//code += "if(!durations.empty())\nvecData.push_back(new WastedTimeData(*it, durations, fromStart, toEnd));\n}\n";
	code += "vecData.push_back(new WastedTimeData(*it, durations, fromStart, toEnd));\n}\n";
}
PENALTY LEFT_SCOPE penFctName:OTHER_ID {
	code += "float totalCost = "+ penFctName->getText() +"(vecData, canvas.totalDuration());\n"; }
RIGHT_SCOPE SEMICOLON RIGHT_SCOPE {
	// code to free memory
	code += "\nfor(unsigned int i=0 ; i<vecData.size() ; ++i)\ndelete vecData[i];\n\n";
	code += "return totalCost;\n}\n\n";
} ;
//********* end wastedTime rule *********


//---------------------------------------------------------------------------------------------------------------
// LEXER CLASS
//---------------------------------------------------------------------------------------------------------------
class HATPLexer extends Lexer;

//******** Lexer options ********
options {
	charVocabulary = '\0'..'\377'; 	// use of ASCII code
	testLiterals = false; 			// no tests needed on substrings
	k = 3; 							// max number of tokens to consider in sequence
	caseSensitive = true;				// case sensitive during token detection
	caseSensitiveLiterals = true;		// case sensitive when comparing token with table of litterals
}

//******** Tokens ********
tokens {
	FACTDATABASE		= "factdatabase";
	HTN			= "HTN";
	DEFINE			= "define";
	ENTITY_TYPE		= "entityType";
	ENTITY_ATTRIBUTES	= "entityAttributes";
	STATIC			= "static";
	DYNAMIC			= "dynamic";
	ATOM			= "atom";
	SET			= "set";
	NUMBER			= "number";
	STRING			= "string";
	BOOL			= "bool";
	ENTITY			= "entity";
	NEW			= "new";
	ACTION			= "action";
	METHOD			= "method";
	EMPTY			= "empty";
	PRECONDITIONS		= "preconditions";
	EFFECTS			= "effects";
	COST			= "cost";
	DURATION		= "duration";
	SUBTASKS		= "subtasks";
	SELECT			= "SELECT";
	SELECTONCE		= "SELECTONCE";
	SELECTORDERED		= "SELECTORDERED";
	OR			= "OR";
	SIZE			= "size";
	IF			= "IF";
	CALL			= "CALL";
	FIND			= "FIND";
	FORALL			= "FORALL";
	EXIST			= "EXIST";
	UNDESIRABLE_STATE	= "undesirableState";
	UNDESIRABLE_SEQUENCE	= "undesirableSequence";
	CONDITIONS		= "conditions";
	SEQUENCE		= "sequence";
	PENALTY			= "penalty";
	EFFORT_BALANCING	= "effortBalancing";
	INTRICACY_CONTROL	= "controlOfIntricacy";
	LINKS			= "links";
	BAD_DECOMPO		= "badDecomposition";
	DECOMPOSITION		= "decomposition";
	NODE			= "node";
	IMMEDIATE_DECOMPO	= "immediateDecomposition";
	DEEP_DECOMPO		= "deepDecomposition";
	NULL_POINTER		= "NULL";
	PRIORITY		= "priority";
	AGENTS			= "agents";
	WASTED_TIME		= "wastedTime";
	TIME_PART		= "timePart";
	MYSELF			= "myself";
	KNOWN			= "known";
	UNKNOWN			= "unknown";
	BMANAGEMENT		= "beliefManagement";
	INFORMATION		= "information";
	CONTRADICTION		= "contradiction";
	QUESTION		= "question";
	COMMACTION		= "commAction";

}

//******** Operators ********
ADD_IN_SET			: "<<=";
REMOVE_FROM_SET			: "=>>";
EQUAL				: "=";
TEST_EQUAL			: "==";
TEST_DIFF			: "!=";
TEST_SET_IN			: ">>";
TEST_SET_NOT_IN			: "!>>";
DOT				: ".";
LEFT_SCOPE			: "{";
RIGHT_SCOPE			: "}";
LEFT_PAREN			: "(";
RIGHT_PAREN			: ")";
COMMA				: ",";
SEMICOLON			: ";";
COLON				: ":";
SUP				: ">";
SUP_EQUAL			: ">=";
INF				: "<";
INF_EQUAL			: "<=";
SUP_TILD			: "~>";
PLUS				: "+";
MINUS				: "- ";
TIMES				: "*";
SLASH				: "/";

//******** Whitespace ******** (must be ignored and not sent to the parser)
WS : ( ' ' | '\t' | '\n' { newline(); } | '\r' )+ { $setType(ANTLR_USE_NAMESPACE(antlr)Token::SKIP); } ;

//******** Comments ******** (like C++ comments, must be ignored and not sent to the parser)
COMMENT : "//" (~('\n' | '\r'))* { $setType(ANTLR_USE_NAMESPACE(antlr)Token::SKIP); } ;

//******** Identifiers ******** (every word except keywords associated to tokens)
ENTITY_ID options { testLiterals = true; } : ('A'..'Z') ('a'..'z' | 'A'..'Z' | '0'..'9'| '_')* ;
OTHER_ID options { testLiterals = true; } : ('a'..'z') ('a'..'z' | 'A'..'Z' | '0'..'9')* ;
NUMBER_ID options { testLiterals = false; } : ('-')? ('0'..'9')+ ('.' ('0'..'9')+)? ;
STRING_ID options{ testLiterals = false; } : ('\"') ('a'..'z' | 'A'..'Z' | '0'..'9')* ('\"') ;


