# Find the Antlr module
# Once found the following variables are set :
#	Antlr_FOUND		True to specify that the module has been found
#
#	Antlr_INCLUDE_DIRS	Path(s) to the headers
#
#	Antlr_LIBRARIES		Link these to use Antlr
#
#	Antlr_EXECUTABLE	Path to the antlr binary

# To help CMake in finding the Antlr module
# one can set the following variables :
# Antlr_INCLUDE To give the path to the folder containing the header files
# or
# AntlrINCLUDE
#
# Antlr_LIB To give the path to (or the containing forlder of) the library file of Antlr
# or
# AntlrLIB
#
# Antlr_BIN To give the full path to (or the containing forlder of) the Antlr binary
# or
# AntlrBIN
#
# You can set any of them using one of the following methods :
#	1) Set it as environnement varialbe (e.g. export, setenv, ...)
#	2) Give it as argument for cmake : cmake -D<var>="...."
#	3) In a CMakeLists.txt, set a variable : set(<var> ...)	

#Varaibles to change if the module changes
set(HEADERS antlr/ANTLRException.hpp antlr/ANTLRUtil.hpp antlr/AST.hpp antlr/ASTArray.hpp antlr/ASTFactory.hpp antlr/ASTNULLType.hpp antlr/ASTPair.hpp antlr/ASTRefCount.hpp antlr/BaseAST.hpp antlr/BitSet.hpp antlr/CharBuffer.hpp antlr/CharInputBuffer.hpp antlr/CharScanner.hpp antlr/CharStreamException.hpp antlr/CharStreamIOException.hpp antlr/CircularQueue.hpp antlr/CommonAST.hpp antlr/CommonASTWithHiddenTokens.hpp antlr/CommonHiddenStreamToken.hpp antlr/CommonToken.hpp antlr/IOException.hpp antlr/InputBuffer.hpp antlr/LLkParser.hpp antlr/LexerSharedInputState.hpp antlr/MismatchedCharException.hpp antlr/MismatchedTokenException.hpp antlr/NoViableAltException.hpp antlr/NoViableAltForCharException.hpp antlr/Parser.hpp antlr/ParserSharedInputState.hpp antlr/RecognitionException.hpp antlr/RefCount.hpp antlr/SemanticException.hpp antlr/String.hpp antlr/Token.hpp antlr/TokenBuffer.hpp antlr/TokenRefCount.hpp antlr/TokenStream.hpp antlr/TokenStreamBasicFilter.hpp antlr/TokenStreamException.hpp antlr/TokenStreamHiddenTokenFilter.hpp antlr/TokenStreamIOException.hpp antlr/TokenStreamRecognitionException.hpp antlr/TokenStreamRetryException.hpp antlr/TokenStreamRewriteEngine.hpp antlr/TokenStreamSelector.hpp antlr/TokenWithIndex.hpp antlr/TreeParser.hpp antlr/TreeParserSharedInputState.hpp antlr/config.hpp)
set(LIBRARIES libantlr.a)
set(EXECUTABLE runantlr)

unset(Antlr_INCLUDE_DIRS)
foreach(header ${HEADERS})
	find_path(${header}_PATH ${header}
	HINTS ${Antlr_INCLUDE} ${AntlrINCLUDE}
	$ENV{antlr_INCLUDE} $ENV{AntlrINCLUDE})
	if(${header}_PATH)
		list(APPEND Antlr_INCLUDE_DIRS ${${header}_PATH})
	else(${header}_PATH)
		list(APPEND MISSING_HEADERS ${header})
	endif(${header}_PATH)
endforeach()

if(Antlr_INCLUDE_DIRS)
	list(REMOVE_DUPLICATES Antlr_INCLUDE_DIRS)
endif(Antlr_INCLUDE_DIRS)

if(MISSING_HEADERS)
	unset(Antlr_INCLUDE_DIRS)
endif(MISSING_HEADERS)

unset(Antlr_LIBRARIES)
if(Antlr_LIB)
	if(EXISTS ${Antlr_LIB} AND NOT IS_DIRECTORY ${Antlr_LIB})
		set(Antlr_LIBRARIES ${Antlr_LIB})
	else(EXISTS ${Antlr_LIB} AND NOT IS_DIRECTORY ${Antlr_LIB})
		message(STATUS "The variable Antlr_LIB points to a none existing file, please corrects its value.")
	endif(EXISTS ${Antlr_LIB} AND NOT IS_DIRECTORY ${Antlr_LIB})
elseif(AntlrLIB)
	if(EXISTS ${AntlrLIB} AND NOT IS_DIRECTORY ${AntlrLIB})
		set(Antlr_LIBRARIES ${AntlrLIB})
	else(EXISTS ${AntlrLIB} AND NOT IS_DIRECTORY ${AntlrLIB})
		message(STATUS "The variable AntlrLIB points to a none existing file, please corrects its value.")
	endif(EXISTS ${AntlrLIB} AND NOT IS_DIRECTORY ${AntlrLIB})
elseif(NOT $ENV{Antlr_LIB} STREQUAL "")
	if(EXISTS $ENV{Antlr_LIB} AND NOT IS_DIRECTORY $ENV{Antlr_LIB})
		set(Antlr_LIBRARIES $ENV{Antlr_LIB})
	else(EXISTS $ENV{Antlr_LIB} AND NOT IS_DIRECTORY $ENV{Antlr_LIB})
		message(STATUS "The environment variable Antlr_LIB points to a none existing file, please corrects its value.")
	endif(EXISTS $ENV{Antlr_LIB} AND NOT IS_DIRECTORY $ENV{Antlr_LIB})
elseif(NOT $ENV{AntlrLIB} STREQUAL "")
	if(EXISTS $ENV{AntlrLIB} AND NOT IS_DIRECTORY $ENV{AntlrLIB})
		set(Antlr_LIBRARIES $ENV{AntlrLIB})
	else(EXISTS $ENV{AntlrLIB} AND NOT IS_DIRECTORY $ENV{AntlrLIB})
		message(STATUS "The environment variable AntlrLIB points to a none existing file, please corrects its valu.")
	endif(EXISTS $ENV{AntlrLIB} AND NOT IS_DIRECTORY $ENV{AntlrLIB})
else(Antlr_LIB)
	find_file(Antlr_LIBRARIES ${LIBRARIES} PATH_SUFFIXES lib
		HINTS ${Antlr_LIB} ${AntlrLIB} ${AntlrLIBDIR}
		$ENV{Antlr_LIB} $ENV{AntlrLIB} $ENV{AntlrLIBDIR})
endif(Antlr_LIB)




unset(Antlr_EXECUTABLE)
if(Antlr_BIN)
	if(EXISTS ${Antlr_BIN} AND NOT IS_DIRECTORY ${Antlr_BIN})
		set(Antlr_EXECUTABLE ${Antlr_BIN})
	else(EXISTS ${Antlr_BIN} AND NOT IS_DIRECTORY ${Antlr_BIN})
		message(STATUS "The variable Antlr_BIN points to a none existing file, please corrects its value.")
	endif(EXISTS ${Antlr_BIN} AND NOT IS_DIRECTORY ${Antlr_BIN})
elseif(AntlrBIN)
	if(EXISTS ${AntlrBIN} AND NOT IS_DIRECTORY ${AntlrBIN})
		set(Antlr_EXECUTABLE ${AntlrBIN})
	else(EXISTS ${AntlrBIN} AND NOT IS_DIRECTORY ${AntlrBIN})
		message(STATUS "The variable AntlrBIN points to a none existing file, please corrects its value.")
	endif(EXISTS ${AntlrBIN} AND NOT IS_DIRECTORY ${AntlrBIN})
elseif(NOT $ENV{Antlr_BIN} STREQUAL "")
	if(EXISTS $ENV{Antlr_BIN} AND NOT IS_DIRECTORY $ENV{Antlr_BIN})
		set(Antlr_EXECUTABLE $ENV{Antlr_BIN})
	else(EXISTS $ENV{Antlr_BIN} AND NOT IS_DIRECTORY $ENV{Antlr_BIN})
		message(STATUS "The environment variable Antlr_BIN points to a none existing file, please corrects its value.")
	endif(EXISTS $ENV{Antlr_BIN} AND NOT IS_DIRECTORY $ENV{Antlr_BIN})
elseif(NOT $ENV{AntlrBIN} STREQUAL "")
	if(EXISTS $ENV{AntlrBIN} AND NOT IS_DIRECTORY $ENV{AntlrBIN})
		set(Antlr_EXECUTABLE $ENV{AntlrBIN})
	else(EXISTS $ENV{AntlrBIN} AND NOT IS_DIRECTORY $ENV{AntlrBIN})
		message(STATUS "The environment variable AntlrBIN points to a none existing file, please corrects its valu.")
	endif(EXISTS $ENV{AntlrBIN} AND NOT IS_DIRECTORY $ENV{AntlrBIN})
else(Antlr_BIN)
	find_program(Antlr_EXECUTABLE ${EXECUTABLE} PATH_SUFFIXES bin 
		HINTS ${Antlr_DIR} ${Antlr_ROOT} ${AntlrROOT}
		$ENV{Antlr_DIR} $ENV{Antlr_ROOT} $ENV{AntlrROOT})
endif(Antlr_BIN)

if(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)
	set(Antlr_FOUND TRUE)
else(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)
	if(NOT Antlr_FIND_QUIETLY)
		if(NOT Antlr_INCLUDE_DIRS)
			message(STATUS "Unable to find Antlr header files!")
		else(NOT Antlr_INCLUDE_DIRS)
			message(STATUS "Header files found.")
		endif(NOT Antlr_INCLUDE_DIRS)
		if(NOT Antlr_LIBRARIES)
			message(STATUS "Unable to find Antlr library files!")
		else(NOT Antlr_LIBRARIES)
			message(STATUS "Library files found.")
		endif(NOT Antlr_LIBRARIES)
		if(NOT Antlr_EXECUTABLE)
			message(STATUS "Unable to find Antlr executable!")
		else(NOT Antlr_EXECUTABLE)
			message(STATUS "Antlr executable found.")
		endif(NOT Antlr_EXECUTABLE)
	endif(NOT Antlr_FIND_QUIETLY)
endif(Antlr_INCLUDE_DIRS AND Antlr_LIBRARIES AND Antlr_EXECUTABLE)

if(Antlr_FOUND)
	if(NOT Antlr_FIND_QUIETLY)
		message(STATUS "Found components for Antlr")
		message(STATUS "Antlr_INCLUDE_DIRS = ${Antlr_INCLUDE_DIRS}")
		message(STATUS "Antlr_LIBRARIES = ${Antlr_LIBRARIES}")
		message(STATUS "Antlr_EXECUTABLE = ${Antlr_EXECUTABLE}")
	endif(NOT Antlr_FIND_QUIETLY)
else(Antlr_FOUND)
	if(Antlr_FIND_REQUIRED)
		set(Antlr_INCLUDE "" CACHE PATH "Paths where to additionally look for Antlr headers")
		set(AntlrINCLUDE "" CACHE PATH "Paths where to additionally look for Antlr headers")
		set(Antlr_LIB "" CACHE PATH "Paths where to additionally look for Antlr library")
		set(AntlrLIB "" CACHE PATH "Paths where to additionally look for Antlr library")
		set(Antlr_BIN "" CACHE PATH "Paths to Antlr binary")
		set(AntlrBIN "" CACHE PATH "Paths to Antlr binary")
		if(NOT Antlr_FIND_QUIETLY)
			message("\nCould not find Antlr!\n")
			message("You can use any of the following variables to help CMake to find the header files :")
			message("	Antlr_INCLUDE or AntlrINCLUDE")
			message("You can use any of the following variables to help CMake to find the library (can be the path or the file):")
			message("	Antlr_LIB or AntlrLIB")
			message("You can use any of the following varaible to give the Antlr binary file (can be the path or the binary file):")
			message("	Antlr_BIN or AntlrBIN")
			message("To set one you can either :")
			message("	- use 'set(<var> ....)' command in the CMakeLists.txt (separator ' ')")
			message("	- use '-D<var>=....' as an option to Cmake (separator ';')")
			message("	- set one as an environment variable (e.g. export in bash, with separator ';')\n")
		endif(NOT Antlr_FIND_QUIETLY)
		message(FATAL_ERROR "Could not find Antlr!")
	endif(Antlr_FIND_REQUIRED)
endif(Antlr_FOUND)

mark_as_advanced (
	Antlr_FOUND
	Antlr_INCLUDE_DIRS
	Antlr_LIBRARIES
	Antlr_EXECUTABLE
)
